/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_MACROBRIDGE_MACROEVENT_H
#define SSTMAC_MACROBRIDGE_MACROEVENT_H

#include <sstmac/simulator/eventhandler.h>
#include <sst/core/event.h>
#include <iostream>

namespace sstmac {

  /**
   * Re-package SST/macroscale event handlers as a SST::Event.
   */
  struct macroevent : public SST::Event {
    boost::shared_ptr<eventhandler> handler_;
  public:
    macroevent(const boost::shared_ptr<eventhandler>& eh) :
      handler_(eh)
    {}
    virtual ~macroevent() {}
    void complete(const timestamp &now) {
      handler_->handle(now);
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_MACROBRIDGE_MACROEVENT_H
