/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


// clos.h: Interface for CLOS networks.
//
// Based on clos.c by Jim Schutt
// Adapted by Curtis Janssen <cljanss@ca.sandia.gov>

#ifndef SSTMAC_NETWORK_CLOS_H
#define SSTMAC_NETWORK_CLOS_H

#include <sstmac/network/network.h>

namespace sstmac {

  /** The clos network generates a full 3-tier CLOS network with 2*n port
      switches.  In the CLOS network, not all vertices are nodes.

      <table border="1">

      <tr><td>Keyword<td>Type<td>Default<td>Description

      <tr><td><tt>n</tt><td>int<td>2<td>Half of the number of ports in each
      switching element.

      </table>

  */
  class clos: public network {
  private:
    int n_;
    void route_hashed(const vertex &v1, const vertex &v2, walk &w) const;
    void init_clos(int n);

  protected:
    //clos(TiXmlHandle);
    clos(int n);

  public:
    static boost::shared_ptr<clos> construct(int half_ports=2);
    virtual ~clos();

    static size_t size(int n);
    void shortest_path(const vertex&v1, const vertex&v2, walk&w) const;
    void route(const vertex &v1, const vertex &v2, walk &w) const;
    int n() const;
    int diameter() const;
    double a() const;
    int b() const;
    bool vertex_symmetric() const;
    std::string name() const;
  };

}

#endif
