/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


// torus.cc: Implementation of torus networks.
//
// Author: Curtis Janssen <cljanss@ca.sandia.gov>

#include <sstream>
#include <sstmac/network/torus.h>
#include <sstmac/network/ring.h>

using std::string;

namespace sstmac {
  
  using boost::shared_ptr;

  // static sstmac::xmlclass::register_class reg("sstmac::torus",
  //                               sstmac::xmlclass::ctor_template<torus>);

  // torus::torus(TiXmlHandle h)
  // {
  //   int n1=4, n2=4, n3=0;

  //   h.ToElement()->QueryIntAttribute("n1",&n1);
  //   h.ToElement()->QueryIntAttribute("n2",&n2);
  //   h.ToElement()->QueryIntAttribute("n3",&n3);

  //   if (n3>0) {
  //       init_torus(n1,n2,n3);
  //     }
  //   else {
  //       init_torus(n1,n2);
  //     }
  // }

  torus::torus()
  {
  }

  void
  torus::init_torus_2D(int x, int y, int nndpersw, int widtharg) 
  {
    dimensions_.resize(2);
    dimensions_.at(0) = x;
    dimensions_.at(1) = y;
    width = widtharg;
    direct_ = (nndpersw == 0);
    nndpersw_ = nndpersw;

    shared_ptr<network> n1 = ring::construct(x,nndpersw);
    shared_ptr<network> n2 = ring::construct(y,(nndpersw>0?1:0));
    assemble(n1,n2);

    std::ostringstream ostr;
    ostr << "Torus" << "("
         << x << "," << y
         << ",nps=" << nndpersw
         << ")";
    name_ = ostr.str();
  }

  void
  torus::init_torus_3D(int x, int y, int z, int nndpersw, int widtharg) 
  {
    dimensions_.resize(3);
    dimensions_.at(0) = x;
    dimensions_.at(1) = y;
    dimensions_.at(2) = z;
    width = widtharg;
    direct_ = (nndpersw == 0);
    nndpersw_ = nndpersw;

    shared_ptr<network> n1 = torus::construct_2D(x,y,nndpersw);
    shared_ptr<network> n2 = ring::construct(z,(nndpersw>0?1:0));
    assemble(n1,n2);

    std::ostringstream ostr;
    ostr << "Torus" << "("
         << x << "," << y << "," << z
         << ",nps=" << nndpersw
         << ")";
    name_ = ostr.str();
  }

  boost::shared_ptr<torus> torus::construct_2D(int n1, int n2,
                                               int nndpersw, int widtharg) {
    boost::shared_ptr<torus> t(new torus);
    t->init_torus_2D(n1,n2,nndpersw,widtharg);
    return t;
  }

  boost::shared_ptr<torus> torus::construct_3D(int n1, int n2, int n3,
                                               int nndpersw, int widtharg) {
    boost::shared_ptr<torus> t(new torus);
    t->init_torus_3D(n1,n2,n3,nndpersw,widtharg);
    return t;
  }
  
  torus::~torus() {
  }

  std::string
  torus::name() const
  {
    return name_;
  }

  int
  torus::edge_width(edge_index e) const
  {
    const lwedge &lwe = g().index_to_edge(e);
    if (is_switch(lwe.from()) && is_switch(lwe.to())) return width;
    return 1;
  }

}
