/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_ACTOREXIT_H
#define SSTMAC_PROCESS_ACTOREXIT_H

#include <boost/shared_ptr.hpp>

namespace sstmac {

  /**
   * Base type for exit reasons in a simulated actor-based application.
   * Also provides common exit reasons.
   */
  class actorexit {
    /// Nested types to indicate common exit reasons.
    class normal;
    class killed;
    class failure;
    /// Nested objects keeping track of exit reasons.
    static boost::shared_ptr<const actorexit> normal_, killed_, failure_;

  public:
    /// Goodbye.
    virtual ~actorexit() {}

    /// Clone the exit reason (this might be excessively anal).
    virtual boost::share_ptr<actorexit> actorclone() const = 0;

    /// Test whether the given exit reason matches this one.
    virtual bool matches(const boost::shared_ptr<const actorexit>& reason) const = 0;

    /// Normal exit.
    static boost::shared_ptr<const actorexit> normal();

    /// Killed for an unspecified reason (node still alive).
    static boost::shared_ptr<const actorexit> killed();

    /// Indicate that node failed or is unreachable (heartbeat monitoring).
    static boost::shared_ptr<const actorexit> failure();
  };
  
} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_ACTOREXIT_H
