/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/actorpattern.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  /* A pattern that matches everything. */
  class actorpattern::anypattern : public actorpattern {
  public:
    /// Goodbye.
    virtual ~anypattern() {}

    /// Match.
    bool matches(const boost::shared_ptr<const actormsg>& msg) const {
      if(msg)
        return true;
      else
        throw nullerror("actorpattern::anypattern:  NULL message is not allowed.");
    }
  };
  
  // The static match for any pattern.
  boost::shared_ptr<const actorpattern> actorpattern::any_;

  // Access the any pattern matcher.
  boost::shared_ptr<const actorpattern> actorpattern::any() {
    if(! any_)
      any_ = boost::shared_ptr<const actorpattern>(new anypattern);
    return any_;
  }

} // end of namespace sstmac.
