/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_ACTORSERVER_CLIENT_LINKREQUEST_H
#define SSTMAC_PROCESS_ACTORSERVER_CLIENT_LINKREQUEST_H

#include <sstmac/process/actorserver-client-linkrespose.h>

namespace sstmac {

  /**
   * An internal message type for actorserver::client.  This object is sent
   * to indicate that a neighbor wants to set up a link with us.
   */
  class actorserver::client::linkrequest :
    public actorserver::client::internalrequest
  {
    /// My contents.
    actorid sender_;
    int32_t linkid_;
    
    /// Why hello there.
    linkrequest(const actorid &sender, int32_t linkid) :
      sender_(sender), linkid_(linkid)
    {}
    
public:
  /// Vroom.
  static boost::shared_ptr<linkrequest> construct(const actorid &snd, int32_t lid) {
    return boost::shared_ptr<linkrequest>(new linkrequest(snd, lid));
  }
    
    /// Adios Chuck!
    virtual ~linkrequest() {}
    
    /// Let's get dangerous (Darkwing Duck!).
    virtual void handle(const shared_ptr<actorserver::client>& parent,
                        const actorid &sender)
    {
      parent->links_.insert(sender);
      parent->send(sender, actorserver::client::linkresponse::construct(linkid));
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_ACTORSERVER_CLIENT_LINKREQUEST_H
