/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_ACTORSERVER_CLIENT_UNMONITORREQUEST_H
#define SSTMAC_PROCESS_ACTORSERVER_CLIENT_UNMONITORREQUEST_H

#include <sstmac/process/actorserver-client-monitorrespose.h>

namespace sstmac {

  /**
   * An internal message type for actorserver::client.  This object is sent
   * to indicate that a neighbor wants to stop monitoring us.
   */
  class actorserver::client::unmonitorrequest :
    public actorserver::client::internalrequest
  {
    /// Contents.
    actorid sender_;
    int32_t monitorid_;
    
    /// Private constructor like for all reference counted types.
    unmonitorrequest(const actorid &sender, int32_t monitorid) :
      sender_(sender), monitorid_(monitorid)
    {}
    
public:
  /// Get a new refererence counted pointer.  Called by the sending node.
  static boost::shared_ptr<unmonitorrequest>
  construct(const actorid &snd, int32_t lid) {
    return boost::shared_ptr<unmonitorrequest>(new unmonitorrequest(snd, lid));
  }
    
    /// Clean up.
    virtual ~unmonitorrequest() {}
    
    /// This is called by the receiving node.
    virtual void handle(const shared_ptr<actorserver::client>& parent,
                        const actorid &sender)
    {
      parent->monitors_.erase(sender);
      parent->send(sender,
                   actorserver::client::monitorresponse::construct(monitorid));
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_ACTORSERVER_CLIENT_UNMONITORREQUEST_H
