/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_ACTORSERVER_CLIENT_MONITORRESPONSE_H
#define SSTMAC_PROCESS_ACTORSERVER_CLIENT_MONITORRESPONSE_H

#include <sstmac/process/actorserver-client-internalrequest.h>
#include <sstmac/process/actorserver-client-monitorhandler.h>

namespace sstmac {

  /**
   * An internal message type used for actorserver::client.  This is a reply
   * from a node saying that our monitor request has been accepted.
   */
  class actorserver::client::monitorresponse :
    public actorserver::client::internalrequest
  {
    /// The monitor request was sent with a unique identifier.
    int32_t monitorid_;
    
    /// As always, the actual constructor is a private affair.
    monitorresponse(int32_t monitorid) : monitorid_(monitorid)
    {}

  public:
    /// Build a dynamically allocated, reference-counted object.
    /// Called by sender.
    static boost::shared_ptr<monitorresponse> construct(int32_t monitorid) {
      return boost::shared_ptr<monitorresponse>(new monitorresponse(monitorid));
    }

    /// Our work here is done.
    virtual ~monitorresponse() {
    }

    /// Handle the completion of this message.  Called by receiver.
    virtual void handle(const shared_ptr<actorserver::client>& parent,
                        const actorid &sender)
    {
      actorserver::client::monitorreq_t::iterator it =
        parent->monitorreq_.find(monitorid_);
      if(monitorid_ != parent->monitorreq_.end()) {
        parent->second->response_received();
      }
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_ACTORSERVER_CLIENT_MONITORRESPONSE_H
