/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_ACTORSERVER_CLIENT_UNLINKREQUEST_H
#define SSTMAC_PROCESS_ACTORSERVER_CLIENT_UNLINKREQUEST_H

#include <sstmac/process/actorserver-client-linkrespose.h>

namespace sstmac {

  /**
   * An internal message type for actorserver::client.  This object is sent
   * to indicate that a neighbor wants remove its link with us.
   */
  class actorserver::client::unlinkrequest :
    public actorserver::client::internalrequest
  {
    /// My contents.
    actorid sender_;
    int32_t linkid_;
    
    /// Pretty standard.
    unlinkrequest(const actorid &sender, int32_t linkid) :
      sender_(sender), linkid_(linkid)
    {}
    
  public:
    /// Get a new object.
    static boost::shared_ptr<unlinkrequest>
    construct(const actorid &snd, int32_t lid) {
      return boost::shared_ptr<unlinkrequest>(new unlinkrequest(snd, lid));
    }
    
    /// Adios Chuck!
    virtual ~unlinkrequest() {}
    
    /// Let's get dangerous (Darkwing Duck!).
    virtual void handle(const shared_ptr<actorserver::client>& parent,
                        const actorid &sender)
    {
      // We use a standard linkresponse for unlink requests.
      parent->links_.erase(sender);
      parent->send(sender, actorserver::client::linkresponse::construct(linkid));
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_ACTORSERVER_CLIENT_UNLINKREQUEST_H
