/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_ACTORTASK_H
#define SSTMAC_PROCESS_ACTORTASK_H

#include <sstmac/process/actorpid.h>
#include <sstmac/process/thread.h>

namespace sstmac {

  /**
   * Base type for processes in a distributed actor-based application.
   */
  class actortask : public thread {
    
  public:
    /// Goodbye.
    virtual ~actortask();

    /// Clone this task.
    /// This should return a new object replicating the state of this
    /// object except for low-level thread state (stack, etc.) which are managed
    /// by the base thread type.
    virtual boost::shared_ptr<actortask> actorclone() const = 0;

    /// Exit signal received from a linked task.
    /// If this method is not overridden, this process will be killed
    /// (taken out of the scheduler) and actormbox::exit will be called
    /// with the current exit reason.
    virtual void exit_signal(const pid &source,
                             const boost::shared_ptr<const actorexit>& reason);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_ACTORTASK_H
