/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_APPLAUNCH_COMPLETION_H
#define SSTMAC_PROCESS_APPLAUNCH_COMPLETION_H

#include <sstmac/process/applaunch.h>
#include <sstmac/simulator/eventhandler.h>

namespace sstmac {

  /**
   * Tell applaunch that one of its applications is exiting.
   */
  class applaunch::completion : public eventhandler {
    /// My parent object.
    boost::shared_ptr<applaunch> parent_;
    
  protected:
    /// Hello.
    completion(const boost::shared_ptr<applaunch>& parent);

  public:
    /// Build an object.
    static boost::shared_ptr<completion>
    construct(const boost::shared_ptr<applaunch>& parent);

    /// Goodbye.
    virtual ~completion() throw();

    /// Signal an event.
    virtual void handle(const timestamp &current_time);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_APPLAUNCH_COMPLETION_H
