/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/key.h>
#include <sstmac/util/alloc.h>
#include <iostream>
#include <stdlib.h>

namespace sstmac {
  
  // Pointer types.
  using boost::shared_ptr;
  using boost::weak_ptr;

  // The key objects are very short-lived, so we don't feel like
  // continually allocating and deallocating those bastards.
  namespace keypool {
    static std::vector<key*> *chunks = 0;

    static key* alloc_chunk() {
      if(! chunks) {
        chunks = new std::vector<key*>;
        chunks->reserve(1024);
      }
      if(! chunks->empty()) {
        key *memptr = chunks->back();
        chunks->pop_back();
        return memptr;
      }
      else {
        key *memptr = sstmac::allocate_aligned<key>();
        return memptr;
      }
    }

    static void free_chunk(key *chunk) {
      chunk->~key();
      chunks->push_back(chunk);
    }

  } // end of namespace keypool.

  boost::shared_ptr<key> key::construct() {
    return boost::shared_ptr<key>
      (new (keypool::alloc_chunk()) key, keypool::free_chunk);
  }

} // end of namespace sstmac.
