/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPIAPP_H
#define SSTMAC_PROCESS_MPIAPP_H

#include <sstmac/process/app.h>

namespace sstmac {

  /// Important types.
  class mpiapi;

  /**
   * A base type for applications that can run via mpi.
   * Intended to be safe for multiple inheritance (for applications
   * that use MPI and some other form of communication).
   */
  class mpiapp : public app {
    boost::shared_ptr<mpiapi> mpi_;

  protected:
    /// Access the MPI API.
    boost::shared_ptr<mpiapi>& mpi() { return mpi_; }

    /// Access the MPI API in a const context.
    boost::shared_ptr<const mpiapi> mpi() const { return mpi_; }

  public:
    /// Goodbye.
    virtual ~mpiapp() {}

    /// Give me another object of this same type, thank you.
    /// This is called by mpilaunch prior to calling init.
    virtual boost::shared_ptr<mpiapp> clone() = 0;

    /// This is your access to the MPI API.
    virtual void init_mpi(const boost::shared_ptr<mpiapi>& mpi) {
      mpi_ = mpi;
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_MPIAPP_H
