/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPIBCASTENGINE_H
#define SSTMAC_PROCESS_MPIBCASTENGINE_H

#include <sstmac/process/mpicollective.h>
#include <sstmac/process/mpicomm.h>
#include <sstmac/process/mpitype.h>
#include <sstmac/process/mpitag.h>

namespace sstmac {

  // Important types.
  class mpiqueue;
  class mpistatus;
  class topologyiterator;
  class payload;
  
  /**
   * A basic mpikernel to provide an MPI_Bcast.
   */
  class mpibcastengine : public mpicollective {
    /// The iterator to whom we entrust decisions on where to send and receive.
    boost::shared_ptr<topologyiterator> iter_;
    /// The size of the data array.
    int count_;
    /// The type we are sending and receiving.
    mpitype type_;
    /// The tag we use for our chatter.
    mpitag tag_;
    /// The payload we are sending.
    boost::shared_ptr<const payload> content_;
    /// My rank.
    mpiid rank_;

    /// The number of pending sends.
    int pending_sends_;
    /// The number of pending receives.
    int pending_recvs_;

    /// Fire off the next set of send and receive operations.
    void sendrecv(const timestamp &now);

  protected:
    /// Hi.
    mpibcastengine(const boost::shared_ptr<topologyiterator>& iter,
                   const boost::shared_ptr<key>& thekey,
                   const boost::shared_ptr<mpiqueue>& queue,
                   int count, const mpitype &type,
                   const mpiid &root, const mpitag &tag, const mpicomm &comm,
                   const boost::shared_ptr<const payload>& content);

    /// Callback method to indicate that a send operation has completed.
    virtual void send_complete(const timestamp &now,
                               int count,
                               const mpitype &type,
                               const mpitag &tag,
                               const mpiid &dest);

    /// Callback method to indicate that a receive operation has completed.
    virtual void recv_complete(const timestamp &now,
                               int count,
                               const boost::shared_ptr<const mpistatus>& stat);

    /// Callback method to indicate that a copy operation has completed.
    virtual void copy_complete(const timestamp &now,
                               int count,
                               const mpitype &type,
                               const mpitag &tag,
                               const mpiid &source);

  public:
    /// Hello.
    static boost::shared_ptr<mpibcastengine>
    construct(const boost::shared_ptr<topologyiterator>& iter,
              const boost::shared_ptr<key>& thekey,
              const boost::shared_ptr<mpiqueue>& queue,
              int count, const mpitype &type,  
              const mpiid &root, const mpitag &tag, const mpicomm &comm,
              const boost::shared_ptr<const payload>& content);

    /// Goodbye.
    virtual ~mpibcastengine() throw();

    /// Start this kernel.
    virtual void start(const timestamp &start_time,
                       const boost::shared_ptr<nodeinterface>& my_node);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MPIBCASTENGINE_H
