/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPICOLLECTIVE_IMMEDIATECOPY_H
#define SSTMAC_PROCESS_MPICOLLECTIVE_IMMEDIATECOPY_H

#include <sstmac/process/mpicollective.h>
#include <sstmac/simulator/eventhandler.h>
#include <sstmac/simulator/nodeinterface.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>
#include <deque>

namespace sstmac {

  /**
   * Utility type to handle concurrent copying out of
   * a shared data array.  Simulates copying data for remote sources.
   */
  class mpicollective::immediatecopy : public eventhandler {
    boost::shared_ptr<mpicollective> parent_;
    int count_;
    mpitype type_;
    mpitag tag_;
    mpiid dest_;

  protected:
    // Hi.
    immediatecopy(const boost::shared_ptr<mpicollective>& parent, int count,
                 const mpitype &type, const mpitag &tag, const mpiid &dest) :
      parent_(parent), count_(count), type_(type), tag_(tag), dest_(dest)
    {}

  public:
    // Hello.
    static boost::shared_ptr<immediatecopy>
    construct(const boost::shared_ptr<mpicollective>& parent, int count,
              const mpitype &type, const mpitag &tag, const mpiid &dest)
    {
      return boost::shared_ptr<immediatecopy>
        (new immediatecopy(parent, count, type, tag, dest));
    }

    // Bye.
    virtual ~immediatecopy() throw() {}

    // Event completion.
    void handle(const timestamp &now) {
      if(debugger::active())
        debug() << "mpbcastengine::immediatecopy[" << this << "]::handle( "
                << now << ")\n";
      parent_->copy_complete(now, count_, type_, tag_, dest_);
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MPICOLLECTIVE_IMMEDIATECOPY_H
