/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicommdata.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  using boost::shared_ptr;
  using boost::weak_ptr;

  //
  // Hello.
  //
  mpicommdata::mpicommdata(mpicommid id,
                           const std::vector<taskid> &peers,
                           const std::vector<nodeid> &nodes) :
    id_(id), peers_(peers), nodes_(nodes)
  {
    if(peers_.size() != nodes_.size())
      throw valueerror("mpicommdata(...):  Number of peers is different "
                       "from number of nodes.");
  }

  //
  // Who am I?
  //
  void mpicommdata::set_self(const boost::shared_ptr<mpicommdata>& self) {
    if(self.get() != this) {
      throw valueerror("mpicommdata::set_self:  'self' pointer does not point "
                       "to 'this' object.");
    }
    self_ = self;
  }

  //
  // This is who I am.
  //
  boost::shared_ptr<mpicommdata> mpicommdata::self() {
    return self_.lock();
  }

  //
  // Build a new object.
  //
  boost::shared_ptr<mpicommdata>
  mpicommdata::construct(const mpicommid &id,
                         const std::vector<taskid> &peers,
                         const std::vector<nodeid> &nodes)
  {
    //std::cerr << "mpicommdata::construct\n";
    boost::shared_ptr<mpicommdata> retval(new mpicommdata(id, peers, nodes));
    retval->set_self(retval);
    return retval;
  }

  //
  // Goodbye.
  //
  mpicommdata::~mpicommdata() {
  }

  //
  // My identifier.
  //
  //const mpicommid& mpicommdata::id() const {
  //  return id_;
  //}

} // end of namespace sstmac
