/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPICOREALLGATHER_H
#define SSTMAC_PROCESS_MPICOREALLGATHER_H

#include <sstmac/process/mpistrategies.h>

namespace sstmac {

  /*
   * Default strategy for allgather operations.
   */
  class mpicoreallgather : public mpiallgatherstrategy {
    /// Hi.
    mpicoreallgather();

  public:
    /// Hello.
    static boost::shared_ptr<mpicoreallgather> construct();

    /// Must be virtual.
    virtual ~mpicoreallgather() throw();

    /// Build a kernel for a allgather operation.
    virtual boost::shared_ptr<mpikernel>
    build(const boost::shared_ptr<key>& thekey,
          const boost::shared_ptr<mpiqueue>& queue,
          int sendcount, const mpitype &sendtype,
          int recvcounts, const mpitype &recvtype,
          const mpitag &tag, const mpicomm &comm,
          const boost::shared_ptr<const payload>& content,
          const boost::shared_ptr<mpimagic>& magic) const;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MPICOREALLGATHER_H
