/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicoreallreduce.h>
#include <sstmac/process/mpirabenseifner.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;

  //
  // Hi.
  //
  mpicoreallreduce::mpicoreallreduce() {
  }

  //
  // Hello.
  //
  shared_ptr<mpicoreallreduce> mpicoreallreduce::construct() {
    return shared_ptr<mpicoreallreduce>(new mpicoreallreduce);
  }

  //
  // Perform a fused allreduce-allreduce operation
  //
  shared_ptr<mpikernel>
  mpicoreallreduce::build(const shared_ptr<key>& thekey, const shared_ptr<mpiqueue>& queue,
                          int count, const mpitype &type,
                          const mpiop &op, const mpitag &tag,
                          const mpicomm &comm, const shared_ptr<const payload>& content,
                          const shared_ptr<mpimagic>& magic) const
  {
    return mpirabenseifner::construct
      (true, thekey, queue, count, type, op, mpiid(-1),
       tag, comm, content, magic);
  }

} // end of namespace sstmac
