/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpicorealltoall.h>
#include <sstmac/process/mpidirectalltoall.h>
#include <sstmac/util/errors.h>
#include <sstmac/util/debug.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;

  //
  // Hi.
  //
  mpicorealltoall::mpicorealltoall() {
  }

  //
  // Hello.
  //
  shared_ptr<mpicorealltoall> mpicorealltoall::construct() {
    return shared_ptr<mpicorealltoall>(new mpicorealltoall);
  }

  //
  // Must be virtual.
  //
  mpicorealltoall::~mpicorealltoall() throw() {
  }

  //
  // Build a kernel.
  //
  shared_ptr<mpikernel>
  mpicorealltoall::build(const shared_ptr<key>& thekey,
                         const shared_ptr<mpiqueue>& queue,
                         int sendcnt, const mpitype &sendtype,
                         int recvcnt, const mpitype &recvtype,
                         const mpitag &tag, const mpicomm &comm,
                         const shared_ptr<const payload>& content,
                         const shared_ptr<mpimagic>& magic) const
  {
    std::vector<int> sendvec(comm.size().id, sendcnt);
    std::vector<int> recvvec(comm.size().id, recvcnt);
    return mpidirectalltoall::construct
      (thekey, queue, sendvec, sendtype, recvvec, recvtype,
       tag, comm, content, magic);
  }

} // end of namespace sstmac
