/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPICORESEND_H
#define SSTMAC_PROCESS_MPICORESEND_H

#include <sstmac/process/mpistrategies.h>

namespace sstmac {

  /// Basic strategy for a "fire-and-forget" send operation.
  class mpicoresend : public mpisendstrategy {
  protected:
    mpicoresend();
  public:
    /// Hello.
    static boost::shared_ptr<mpicoresend> construct();
    /// Goodbye.
    virtual ~mpicoresend() throw();
    /// Basic nonblocking send.  A simulator may opt to delay completion of
    /// this operation depending on message size.
    virtual boost::shared_ptr<mpikernel>
    build(const boost::shared_ptr<key>& thekey,
          const boost::shared_ptr<mpiqueue>& queue,
          int count, const mpitype &type, const mpiid &dest,
          const mpitag &tag, const mpicomm &comm,
          const boost::shared_ptr<const payload>& content,
          const boost::shared_ptr<mpimagic>& magic) const;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MPICORESEND_H
