/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpiprobe.h>
#include <sstmac/process/mpimessage.h>
#include <sstmac/process/scheduler.h>
#include <sstmac/process/key.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;

  //
  // Hello.
  //
  mpiprobe::mpiprobe(const shared_ptr<scheduler>& sched,
                     const shared_ptr<key>& thekey) :
    done_(false), sched_(sched), key_(thekey)
  {}

  //
  // Goodbye.
  //
  mpiprobe::~mpiprobe() throw() {
  }

  //
  // Hi.
  //
  shared_ptr<mpiprobe> mpiprobe::construct(const shared_ptr<scheduler>& sched,
                                           const shared_ptr<key>& thekey)
  {
    return shared_ptr<mpiprobe>(new mpiprobe(sched, thekey));
  }

  //
  // Set the status associated with this probe request.
  //
  void mpiprobe::set_mpistatus(const shared_ptr<mpistatus>& status) {
    status_ = status;
  }

  //
  // Get the status associated with this probe request.
  //
  shared_ptr<mpistatus> mpiprobe::get_mpistatus() {
    return status_;
  }

  //
  // Handle.
  //
  void mpiprobe::handle(const timestamp &) {
    done_ = true;
    if(sched_ != 0)
      sched_->unblock(key_);
  }

} // end of namespace sstmac
