/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/mpiqueue-proberequest.h>
#include <sstmac/process/mpiprobe.h>
#include <sstmac/process/mpistatus.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;
  using boost::weak_ptr;

  //
  // Hi there.
  //
  mpiqueue::proberequest::proberequest(const mpicomm &comm,
                                       const mpiid &source, const mpitag &tag,
                                       const shared_ptr<mpiprobe>& completion) :
    myid_(comm.id()), source_(source), tag_(tag), completion_(completion)
  {}

  //
  // Hello.
  //
  shared_ptr<mpiqueue::proberequest>
  mpiqueue::proberequest::construct(const mpicomm &comm, const mpiid &source,
                                    const mpitag &tag,
                                    const shared_ptr<mpiprobe>& completion)
  {
    return shared_ptr<proberequest>
      (new proberequest(comm, source, tag, completion));
  }

  //
  // Test whether we match a given message.
  //
  bool mpiqueue::proberequest::
  matches(const boost::shared_ptr<mpimessage>& message) const {
    //std::cerr << "myid_ == " << myid_ << "\n";
    //std::cerr << "message->commid() == " << message->commid() << "\n";
    mpicommid inid = message->commid();
    bool sameid = (myid_ == inid);
    return (sameid && 
            source_.matches(message->source()) &&
            tag_.matches(message->tag()));
  }

  //
  // Consider this request complete.
  //
  void mpiqueue::proberequest::complete(const timestamp &current_time,
                                        const shared_ptr<mpimessage>& message)
  {
    if(completion_ != 0) {
      completion_->set_mpistatus(message->build_status());
      completion_->handle(current_time);
    }
  }

} // end of namespace sstmac.
