/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPIQUEUE_PROBEREQUEST_H
#define SSTMAC_PROCESS_MPIQUEUE_PROBEREQUEST_H

#include <sstmac/process/mpiqueue.h>

namespace sstmac {

  /**
   * Nested type to handle individual probe requests.
   */
  class mpiqueue::proberequest {
    /// The parameters we will be matching on.
    mpicommid myid_;
    mpiid source_;
    mpitag tag_;
    /// The object waiting for our successful completion.
    /// If this pointer is null, nobody will be told about our raging success.
    boost::shared_ptr<mpiprobe> completion_;

  protected:
    /// Hi there.
    proberequest(const mpicomm &comm, const mpiid &source, const mpitag &tag,
                 const boost::shared_ptr<mpiprobe>& completion);

  public:
    /// Hello.
    static boost::shared_ptr<proberequest>
    construct(const mpicomm &comm, const mpiid &source, const mpitag &tag,
              const boost::shared_ptr<mpiprobe>& completion);

    /// Test whether we match a given message.
    bool matches(const boost::shared_ptr<mpimessage>& message) const;

    /// Consider this request complete.
    void complete(const timestamp &current_time,
                  const boost::shared_ptr<mpimessage>& message);
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_MPIQUEUE_PROBEREQUEST_H
