/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPIRECVENGINE_H
#define SSTMAC_PROCESS_MPIRECVENGINE_H

#include <sstmac/process/mpikernel.h>
#include <sstmac/process/mpirecv.h>
#include <sstmac/process/mpimessage.h>

namespace sstmac {

  // Important types.
  class mpiqueue;

  /*
   * The engine (kernel) that performs a non-synchronous MPI receive.
   */
  class mpirecvengine : virtual public mpikernel, virtual public mpirecv {
    /// This object.
    boost::weak_ptr<mpirecvengine> self_;
    boost::shared_ptr<mpiqueue> queue_;
    int count_;
    mpitype type_;
    mpiid source_;
    mpitag tag_;
    mpicomm comm_;
    mpimessage::category cat_;
    
  protected:
    /// Constructor.
    mpirecvengine(const boost::shared_ptr<key>& thekey,
                  const boost::shared_ptr<mpiqueue>& queue,
                  int count, const mpitype &type, const mpiid &source,
                  const mpitag &tag, const mpicomm &comm,
                  mpimessage::category cat);

    /// Who am I?
    void set_self(const boost::shared_ptr<mpirecvengine>& self);

    /// This is who I am.
    boost::shared_ptr<mpirecvengine> get_self();

  public:
    /// Build me a sharable object.
    static boost::shared_ptr<mpirecvengine>
    construct(const boost::shared_ptr<key>& thekey,
              const boost::shared_ptr<mpiqueue>& queue,
              int count, const mpitype &type, const mpiid &source,
              const mpitag &tag, const mpicomm &comm, mpimessage::category cat);

    /// Goodbye.
    virtual ~mpirecvengine() throw();

    /// This is the method from the kernel.
    virtual void start(const timestamp &start_time,
                       const boost::shared_ptr<nodeinterface>& my_node);

    /// Grab the completion message.
    virtual void handle(const timestamp &current_time);
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESS_MPIRECVENGINE_H
