/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_MPITYPE_H
#define SSTMAC_PROCESS_MPITYPE_H

#include <sstmac/simulator/timestamp.h>
#include <string>
#include <iosfwd>

namespace sstmac {

  /// MPI datatypes.
  struct mpitype {
    std::string label;
    int size;
    explicit mpitype(const std::string &labelit = "", const int sizeit = 0);

    static const mpitype mpi_null;
    static const mpitype mpi_char;
    static const mpitype mpi_unsigned_char;
    static const mpitype mpi_signed_char;
    static const mpitype mpi_wchar;
    static const mpitype mpi_unsigned_long_long;
    static const mpitype mpi_lb;
    static const mpitype mpi_ub;
    static const mpitype mpi_byte;
    static const mpitype mpi_short;
    static const mpitype mpi_unsigned_short;
    static const mpitype mpi_int;
    static const mpitype mpi_unsigned;
    static const mpitype mpi_long;
    static const mpitype mpi_unsigned_long;
    static const mpitype mpi_float;
    static const mpitype mpi_double;
    static const mpitype mpi_long_double;
    static const mpitype mpi_long_long_int;
    static const mpitype mpi_long_long;
    static const mpitype mpi_packed;
    //static const mpitype mpi_lb;
    //static const mpitype mpi_ub;
    static const mpitype mpi_float_int;
    static const mpitype mpi_double_int;
    static const mpitype mpi_long_int;
    static const mpitype mpi_short_int;
    static const mpitype mpi_2int;
    static const mpitype mpi_long_double_int;
    static const mpitype mpi_complex;
    static const mpitype mpi_double_complex;
    static const mpitype mpi_logical;
    static const mpitype mpi_real;
    static const mpitype mpi_double_precision;
    static const mpitype mpi_integer;
    static const mpitype mpi_2integer;
    static const mpitype mpi_2complex;
    static const mpitype mpi_2double_complex;
    static const mpitype mpi_2real;
    static const mpitype mpi_2double_precision;
    static const mpitype mpi_character;
  };
  bool operator==(const mpitype &a, const mpitype &b);
  bool operator!=(const mpitype &a, const mpitype &b);
  std::ostream& operator<<(std::ostream &os, const mpitype &type);

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_MPITYPE_H
