/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/process/multicomplete-handler.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  // Pointer types.
  using boost::shared_ptr;
  using boost::weak_ptr;

  //
  // Hello.
  //
  multicomplete::handler::
  handler(const shared_ptr<multicomplete>& parent, int index) :
    parent_(parent), index_(index)
  {}

  //
  // Who am I?
  void multicomplete::handler::set_self(const shared_ptr<handler>& self) {
    if(self.get() != this) {
      throw valueerror("multicomplete::handler::set_self:  'self' pointer "
                       "does not point to 'this' object.");
    }
    self_ = self;
  }

  //
  // This is who I am.
  //
  shared_ptr<multicomplete::handler> multicomplete::handler::get_self() {
    return self_.lock();
  }

  //
  // New and improved.
  //
  shared_ptr<multicomplete::handler> multicomplete::handler::
  construct(const shared_ptr<multicomplete>& parent, int index) {
    shared_ptr<handler> retval(new handler(parent, index));
    retval->set_self(retval);
    return retval;
  }

  //
  // Goodbye.
  //
  multicomplete::handler::~handler() throw() {
  }

  //
  // All done.
  //
  void multicomplete::handler::handle(const timestamp &current_time) {
    parent_->handler_done(current_time, this->get_self(), index_);
  }


} // end of namespace sstmac
