/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_SIMPLEMAILSERVER_DATA_H
#define SSTMAC_PROCESS_SIMPLEMAILSERVER_DATA_H

#include <sstmac/process/simplemailserver.h>
#include <sstmac/simulator/networkdata.h>

namespace sstmac {

  /**
   * Payload sent by a simplemailserver.
   */
  class simplemailserver::data : public networkdata {
    /// This object.
    boost::weak_ptr<data> self_;
    
    /// WHAT -- is your name?
    actorid source_;
    /// WHAT -- is your quest?
    actorid dest_;
    /// WHAT -- is your weight?
    int64_t bytes_;
    /// Carry on.
    boost::shared_ptr<actormsg> payload_;

    /// Creationism.
    data(actorid src, actorid dst, int64_t mlen, const boost::shared_ptr<actormsg>& pload) :
      source_(src), dest_(dst), bytes_(mlen), payload_(pload)
    {}

  public:
    /// And off we go.
    static boost::shared_ptr<data> construct(actorid src, actorid dst, int64_t mlen,
                                             const boost::shared_ptr<actormsg>& pload)
    {
      boost::shared_ptr<data> rv(new data(src, dst, mlen, pload));
      rv->self_ = rv;
      return rv;
    }

    /// Goodbye.
    virtual ~data() throw() {}

    const actorid& source() const { return source_; }
    const actorid& dest() const { return dest_; }
    int64_t bytes() const { return bytes_; }
    boost::shared_ptr<actormsg> payload() const { return payload_; }
    
    /// required by networkdata.
    const taskid& source_task() const {
      static const taskid zero(0);
      return zero;
    }

    /// required by networkdata.
    const taskid& dest_task() const {
      static const taskid zero(0);
      return zero;
    }

    /// This method is required by the networkdata.
    /// We know that we don't reuse data, so we just return this object.
    boost::shared_ptr<networkdata> complement() const {
      return self_.lock();
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_SIMPLEMAILSERVER_DATA_H
