/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_SIMPLEMBOXAPP_H
#define SSTMAC_PROCESS_SIMPLEMBOXAPP_H

#include <sstmac/process/app.h>
#include <sstmac/process/simplembox.h>

namespace sstmac {

  /**
   * Base type for applications that do simple mailbox-based messaging.
   */
  class simplemboxapp : public app {
    /// Keep the mailbox here.
    boost::shared_ptr<simplembox> mbox_;

  protected:
    /// Access our mailbox.
    boost::shared_ptr<simplembox> mbox() { return mbox_; }
    
    /// Access our mailbox.
    boost::shared_ptr<const simplembox> mbox() const { return mbox_; }
    
  public:
    /// Goodbye.
    virtual ~simplemboxapp() {}

    /// Get us a deep copy of this object.
    virtual boost::shared_ptr<simplemboxapp> clone() const = 0;

    /// Initialize our mailbox in preparation for a run.
    /// If you override this you should still call simplemboxapp::init_mbox.
    virtual void init_mbox(const boost::shared_ptr<simplembox>& the_mbox) {
      mbox_ = the_mbox;
    }

    /// The run method still needs to be overridden by the app type.
    using app::run;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_SIMPLEMBOXAPP_H
