/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESS_SIMPLEMBOXLAUNCH_H
#define SSTMAC_PROCESS_SIMPLEMBOXLAUNCH_H

#include <sstmac/process/applaunch.h>

namespace sstmac {
  
  /// Important types.
  class simplemboxapp;
  
  /**
   * Launch processes that rely on simplembox for communications.
   */
  class simplemboxlaunch : public applaunch {
    /// The object that gets cloned for each process.
    boost::shared_ptr<simplemboxapp> peer_template_;

    /// Actual object construction.
    simplemboxlaunch(const boost::shared_ptr<simplemboxapp>& ptype);

  public:
    /// Create a new launcher.
    static boost::shared_ptr<simplemboxlaunch>
    construct(const boost::shared_ptr<simplemboxapp>& ptype);

    /// Clean up.
    virtual ~simplemboxlaunch() throw();
    
    /// What are we?
    const std::string& description() const;
    
    /// Fire this object off.
    void run(const timestamp &current_time,
             const boost::shared_ptr<environment>& my_world);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_PROCESS_SIMPLEMBOXLAUNCH_H
