/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/processor/eventdata.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  ///
  /// Private constructor
  ///
  eventdata::eventdata() {};

  ///
  /// Who am I?
  ///
  void 
  eventdata::set_self(const boost::shared_ptr<eventdata>& self) {
    if(self.get() != this) {
      throw valueerror("eventdata::set_self:  'self' pointer does "
                       "not refer to 'this' object.");
    }
    self_ = self;
  }

  ///
  /// Construct a new eventdata object.
  ///
  boost::shared_ptr<eventdata> 
  eventdata::construct() 
  {
    boost::shared_ptr<eventdata> 
      retval(new eventdata());
    retval->set_self(retval);
   return retval;
  }

  ///
  /// Get event data
  ///
  uint64_t
  eventdata::get_event(std::string key)
  {
    if( data.find(key) == data.end() )
      throw valueerror("event key not present");
    return data[key]; 
  }

  ///
  /// Test for event key presence
  ///
  bool
  eventdata::has_event(std::string key)
  {
    if( data.find(key) == data.end() )
      return false;
    return true;
  }

} // end of namespace sstmac
