/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/processor/procscheduler-loadhandler.h>
#include <sstmac/processor/procscheduler-loadmonitor.h>
#include <sstmac/simulator/eventmanager.h>

namespace sstmac {

  // Protected constructor
  procscheduler::loadhandler::
  loadhandler(int64_t compute_id,
              const boost::shared_ptr<loadmonitor>& monitor, 
              const boost::shared_ptr<eventmanager>& mgr,
              const boost::shared_ptr<eventhandler>& completion ):
    compute_id_(compute_id), monitor_(monitor), manager_(mgr),
    completion_(completion)
  { 
  }

  // Public constructor
  boost::shared_ptr<procscheduler::loadhandler>
  procscheduler::loadhandler::
  construct(int64_t compute_id,
            const boost::shared_ptr<loadmonitor>& monitor,
            const boost::shared_ptr<eventmanager>& mgr,
            const boost::shared_ptr<eventhandler>& completion)
  {
    return boost::shared_ptr<loadhandler>
      (new loadhandler(compute_id,monitor,mgr,completion));
  }

  //
  // Goodbye.
  //
  procscheduler::loadhandler::~loadhandler() throw()
  {
  }

  //
  // Event.
  //
  void procscheduler::loadhandler::handle(const timestamp &now)
  {
    if( monitor_->is_complete(compute_id_) )
      manager_->schedule(manager_->now(),completion_);
    else
      manager_->schedule( monitor_->next_end_time(compute_id_),
                          loadhandler::construct(compute_id_,monitor_,
                                                 manager_,completion_) );
  }

} // end of namespace sstmac.
