/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_PROCESSOR_PROCSCHEDULER_LOADHANDLER_H
#define SSTMAC_PROCESSOR_PROCSCHEDULER_LOADHANDLER_H

#include <sstmac/processor/procscheduler.h>
#include <sstmac/simulator/eventhandler.h>

namespace sstmac {

  /**
   * Processor scheduler's event handler
   */
  class procscheduler::loadhandler : public eventhandler {

  private:
    int64_t compute_id_;
    boost::shared_ptr<loadmonitor> monitor_;
    boost::shared_ptr<eventmanager> manager_;
    boost::shared_ptr<eventhandler> completion_;
    
  protected:
    /// Protected constructor
    loadhandler(int64_t compute_id,
                const boost::shared_ptr<loadmonitor>& monitor,
                const boost::shared_ptr<eventmanager>& mgr,
                const boost::shared_ptr<eventhandler>& completion );

  public:
    /// Public constructor
    static boost::shared_ptr<loadhandler>
    construct(int64_t compute_id,
              const boost::shared_ptr<loadmonitor>& monitor,
              const boost::shared_ptr<eventmanager>& mgr,
              const boost::shared_ptr<eventhandler>& completion );

    /// Goodbye.
    virtual ~loadhandler() throw();

    /// Event.
    void handle(const timestamp &completion_time);
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_PROCESSOR_PROCSCHEDULER_LOADHANDLER_H
