/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_EVENTMANAGER_H
#define SSTMAC_SIMULATOR_EVENTMANAGER_H

#include <sstmac/simulator/timestamp.h>
#include <boost/shared_ptr.hpp>

namespace sstmac {

  // Forward declaration of other relevant types.
  class eventhandler;

  /**
   * Base type for network eventmanagers.
   */
  class eventmanager {
    friend class manager;
    timestamp now_;
  protected:
    void set_now(const timestamp &ts) {
      now_ = ts;
    }
  public:
    /// Goodbye.
    virtual ~eventmanager() {}

    /// Clear all events and set time back to a zero of your choice.
    /// This call shall not be permitted while the event manager is running.
    virtual void clear(const timestamp &zero_time = timestamp(0)) = 0;

    /// Run the eventmanager.
    /// The eventmanager shall return control when no more messages remain.
    virtual void run() = 0;

    /// Get the current eventmanager time.
    /// This call must work regardless of whether the eventmanager is currently
    /// running or not.  After a run, the call should return the time at which
    /// the last event was completed.
    virtual const timestamp& now() const { return now_; }

    /// Set off the given eventhandler at the given time.
    virtual void schedule(const timestamp &start_time,
                          const boost::shared_ptr<eventhandler> &handler) = 0;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_EVENTMANAGER_H
