/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_INDEXSTRATEGY_H
#define SSTMAC_SIMULATOR_INDEXSTRATEGY_H

#include <sstmac/simulator/nodeid.h>
#include <vector>

namespace sstmac {

  /**
   * Base class for strategies regarding how to sequentially number nodes
   * in a parallel simulation.  
   */
  class indexstrategy {
  public:
    virtual ~indexstrategy() throw() {}
    
    /// Given a vector of nodes of length nodes.size() in which each
    /// node appears only once
    /// and a value of ppn processes per node,
    /// provide a list of length
    /// nodes.size()*ppn refering to the nodes on which each run resides.
    virtual void allocate(const std::vector<nodeid> &nodes,
                          int ppn,
                          std::vector<nodeid> &result) = 0;
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_INDEXSTRATEGY_H
