/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_LAUNCHHANDLER_H
#define SSTMAC_SIMULATOR_LAUNCHHANDLER_H

#include <sstmac/simulator/eventhandler.h>
#include <sstmac/simulator/launcher.h>

namespace sstmac {

  /**
   * Event handler that fires off a launcher.
   */
  class launchhandler : public eventhandler {
  private:
    boost::shared_ptr<launcher> launcher_;
    boost::shared_ptr<environment> world_;
  protected:
    /// Hello
    launchhandler(const boost::shared_ptr<launcher> &the_launcher,
                  const boost::shared_ptr<environment> &the_world) :
      launcher_(the_launcher), world_(the_world)
    {}

  public:
    /// Make a new object.
    static boost::shared_ptr<launchhandler>
    construct(const boost::shared_ptr<launcher> &the_launcher,
              const boost::shared_ptr<environment> &the_world)
    {
      return boost::shared_ptr<launchhandler>
        (new launchhandler(the_launcher, the_world));
    }

    /// Goodbye
    virtual ~launchhandler() throw() {}

    /// Complete the request.
    void handle(const timestamp &completion_time) {
      launcher_->run(completion_time, world_);
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SIMULATOR_LAUNCHHANDLER_H
