/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_LINEARLATENCY_H
#define SSTMAC_SIMULATOR_LINEARLATENCY_H

#include <sstmac/simulator/latencymodel.h>
#include <boost/shared_ptr.hpp>

namespace sstmac {

  /**
   * Assume that latency is linear down to some base value.
   * This is a remarkably common latency profile for both memory and network.
   */
  class linearlatency : public latencymodel {
    /// Slope and intersect.
    int64_t slope_;
    timestamp intersect_;

    /// This is the way we wash our hair, wash our hair, wash our hair.
    /// This is the way we wash our hair, so early in the morning.
    linearlatency(int64_t slope, timestamp intersect) :
      slope_(slope), intersect_(intersect)
    {}

  public:
    /// With apologies for the slightly insane units:
    /// \param slope      curve slope given in picoseconds per byte
    /// \param intersect  curve intersect given in time units.
    static boost::shared_ptr<linearlatency>
    construct(int64_t slope, timestamp intersect)
    {
      return boost::shared_ptr<linearlatency>
        (new linearlatency(slope, intersect));
    }
    
    /// Auf wiedersehen.
    virtual ~linearlatency() {}

    /// But I hate waiting!
    virtual timestamp latency(data payload) const {
      timestamp trans = timestamp::exact_psec(slope_ * payload.get_data(data::B));
      return (trans + intersect_);
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_LINEARLATENCY_H
