/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_NETSHARING_H
#define SSTMAC_SIMULATOR_NETSHARING_H

#include <iostream>

namespace sstmac {

  /// The network sharing models recognized by shared-bandwidth interconnects.
  enum netsharing {
    OVERCOMMIT, // Turn the network into a crossbar.
    THROTTLE_OVERCOMMITS,
    ORDERED
  };

  inline std::ostream& operator<<(std::ostream &os, netsharing ns) {
    os << "netsharing(";
    switch(ns) {
    case OVERCOMMIT: os << "OVERCOMMIT"; break;
    case THROTTLE_OVERCOMMITS: os << "THROTTLE_OVERCOMMITS"; break;
    case ORDERED: os << "ORDERED"; break;
    default: os << "INVALID_VALUE=" << int(ns);
    }
    os << ")";
    return os;
  }

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_NETSHARING_H
