/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_NETWORKCALLTRAP_H
#define SSTMAC_SIMULATOR_NETWORKCALLTRAP_H

#include <sstmac/simulator/networkcall.h>
#include <sstmac/simulator/timestamp.h>
#include <sstmac/simulator/nodeid.h>
#include <sstmac/simulator/ppid.h>
#include <boost/shared_ptr.hpp>

namespace sstmac {

  // Important types.
  class networkdata;
  class eventhandler;

  /**
   * Trap network calls from a node.  Allows node to be initialized without a network.
   */
  class networkcalltrap: public networkcall {

  public:

    /// Trap a send.
    virtual void send(const timestamp &start_time,
                      const ppid &the_process,
                      const nodeid &the_receiver,
                      const boost::shared_ptr<networkdata> &the_payload,
                      const boost::shared_ptr<eventhandler> &the_sendevent);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_NETWORKCALLTRAP_H
