/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_NODEID_H
#define SSTMAC_SIMULATOR_NODEID_H

#include <iostream>

namespace sstmac {

  /**
   * A ligthweight wrapper for a node index.
   * Done to avoid annoying mistakes where index values are mixed up.
   * 
   * I can't wait for C++ to add opaque typedefs.
   */
  struct nodeid {
    int id;
    explicit nodeid(int v = -1) : id(v) {}

    inline nodeid& operator++() {
      ++id;
      return *this;
    }
    inline nodeid operator++(int) {
      nodeid other(*this);
      ++id;
      return other;
    }
    nodeid operator=(const nodeid &a) {
      id = a.id;
      return *this;
    }
  };

  inline bool operator==(const nodeid &a, const nodeid &b) {
    return (a.id == b.id);
  }
  inline bool operator!=(const nodeid &a, const nodeid &b) {
    return (a.id != b.id);
  }
  inline bool operator<(const nodeid &a, const nodeid &b) {
    return (a.id < b.id);
  }
  inline bool operator<=(const nodeid &a, const nodeid &b) {
    return (a.id <= b.id);
  }
  inline bool operator>(const nodeid &a, const nodeid &b) {
    return (a.id > b.id);
  }
  inline bool operator>=(const nodeid &a, const nodeid &b) {
    return (a.id >= b.id);
  }
  inline std::ostream& operator<<(std::ostream &os, const nodeid &n) {
    return (os << "nodeid(" << n.id << ")");
  }

} // end of namespace sstmac

#endif // ! SSTMAC_SIMULATOR_NODEID_H
