/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_ROUNDROBININDEXING_H
#define SSTMAC_SIMULATOR_ROUNDROBININDEXING_H

#include <sstmac/simulator/indexstrategy.h>
#include <boost/shared_ptr.hpp>

namespace sstmac {

  /**
   * An index strategy that allocates indices using a round robin.
   */
  class roundrobinindexing : public indexstrategy {
  protected:
    roundrobinindexing();

  public:
    static boost::shared_ptr<roundrobinindexing> construct();

    virtual ~roundrobinindexing() throw();
    
    /// Perform round robin allocation.
    /// This method does not test the uniqueness of input nodes.
    /// \param nodes is the list of unique nodes to be used for the allocation
    /// \param ppn is the number of processes allocated on each node.
    /// \param result is the resulting vector of length nodes.size()*ppn
    /// \throw valueerror if nodes.empty()
    /// \throw valueerror if ppn <= 0
    virtual void allocate(const std::vector<nodeid> &nodes,
                          int ppn,
                          std::vector<nodeid> &result);

  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SIMULATOR_ROUNDROBININDEXING_H
