/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SIMULATOR_TASKLIST_H
#define SSTMAC_SIMULATOR_TASKLIST_H

#include <sstmac/simulator/taskid.h>
#include <sstmac/simulator/nodeid.h>
#include <boost/shared_ptr.hpp>
#include <vector>

namespace sstmac {

  /**
   * Map between task indices and node locations.
   */
  class tasklist {
    /// The actual mapping.
    std::vector<nodeid> loc_;

  protected:
    /// Hello.
    tasklist(const std::vector<nodeid> &loc);

  public:
    /// Hi.
    static boost::shared_ptr<tasklist>
    construct(const std::vector<nodeid> &loc);

    /// The number of tasks.
    taskid size() const;

    /// The location of the given task.
    const nodeid& node(const taskid &index) const;

    /// Access the raw vector of all nodeids (mainly for internal use).
    const std::vector<nodeid> nodes() const {
      return loc_;
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SIMULATOR_TASKLIST_H
