/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_MINIMD_MINIMD_ATOM_H
#define SSTMAC_SKELETON_MINIMD_MINIMD_ATOM_H

#include <sstmac/skeleton/minimd/minimd.h>
#include <vector>

namespace sstmac {

  // Ignored types.
  class runinterpolator;
  class mpiapi;

  /**
   * Take over the responsibilities of class Atom in miniMD.
   */
  class minimd::atom {
  public:
    struct Box {
      double xprd, yprd, zprd;
      double xlo, xhi;
      double ylo, yhi;
      double zlo, zhi;
      void set_prd(double x, double y, double z) {
        xprd = x; yprd = y; zprd = z;
      }
    };

    int natoms;
    std::vector<int> neighatoms; // Added to keep track of everybodies nlocal.
    int nlocal;
    int nmax;
    Box box;

    void init(const boost::shared_ptr<runinterpolator>&, const boost::shared_ptr<mpiapi>&) {
      // all atom:: operations are rolled into runtime overhead for other calls.
    }
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SKELETON_MINIMD_MINIMD_ATOM_H
