/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_MPIPARSERS_RSQALLOCATION_H
#define SSTMAC_SKELETON_MPIPARSERS_RSQALLOCATION_H

#include <sstmac/simulator/allocationstrategy.h>


namespace sstmac {

  /// Important types.
  class rsqmetatrace;
  class torus;

  /**
   * An allocation strategy specific to runs on RSQ.
   * Target node indices are read from the given meta-information.
   */
  class rsqallocation : public allocationstrategy {
    /// The metadata.
    boost::shared_ptr<rsqmetatrace> meta_;

    /// Our mesh.
    boost::shared_ptr<sstmac::torus> torus_;

  protected:
    /// A new allocation strategy.
    rsqallocation(const boost::shared_ptr<rsqmetatrace>& meta,
                  const boost::shared_ptr<sstmac::torus>& tor);

  public:
    /// Hello.
    static boost::shared_ptr<rsqallocation>
    construct(const boost::shared_ptr<rsqmetatrace>& meta,
              const boost::shared_ptr<sstmac::torus>& tor);

    /// Goodbye.
    virtual ~rsqallocation() throw();

    /// The main allocate call.
    /// @param nnode number of nodes requested
    /// @param allocation (out) returns the nodes that have been allocated
    /// @param available (inout) the available set of nodes
    /// @param allocated (inout) the allocated set of nodes
    /// Nodes are removed from available and put into allocated.
    /// If not enough nodes are available, then this will throw.
    virtual void allocate(int nnode,
                          std::vector<nodeid> &allocation,
                          std::set<nodeid> &available,
                          std::set<nodeid> &allocated);
  };

} // end of namespace sstmac

#endif // ! SSTMAC_SKELETON_MPIPARSERS_RSQALLOCATION_H
