/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_MPIPARSERS_RSQINDEXING_H
#define SSTMAC_SKELETON_MPIPARSERS_RSQINDEXING_H

#include <sstmac/simulator/blockindexing.h>
#include <sstmac/util/errors.h>

namespace sstmac {

  /**
   * Index strategy for RSQ (if you don't know what this is,
   * consider yourself lucky).
   */
  class rsqindexing : public indexstrategy {
    /// rsqindexing is really just blockindexing with ppn=1
    shared_ptr<blockindexing> idx_;

    /// Hi.
    rsqindexing() :
      idx_(blockindexing::construct())
    {}

  public:
    /// Build.
    static boost::shared_ptr<rsqindexing> construct() {
      return boost::shared_ptr<rsqindexing>(new rsqindexing);
    }

    virtual ~rsqindexing() throw() {
    }

    /// Perform allocation.
    virtual void allocate(const std::vector<nodeid> &nodes,
                          int ppn,
                          std::vector<nodeid> &result)
    {
      if(ppn != 1)
        throw valueerror("rsqindexing:  ppn must be 1.");
      idx_->allocate(nodes, 1, result);
    }
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SKELETON_MPIPARSERS_RSQINDEXING_H
