/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SKELETON_OSUBENCHMARKS_OSULAT_H
#define SSTMAC_SKELETON_OSUBENCHMARKS_OSULAT_H

#include <sstmac/process/mpiapp.h>
#include <string>

namespace sstmac {

/**
 * Reimplementation of the OSU MPI Latency Test v.3.1.1 for SST/macro.
 *
 * The original OSU MPI Latency Test is
 * Copyright (C) 2002-2008 the Network-Based Computing Laboratory
 * (NBCL), The Ohio State University.
 * Contact: Dr. D. K. Panda (panda@cse.ohio-state.edu)
 * Distributed under the BSD license.
 */
  class osulat : public mpiapp {
  private:
    /// The file name we will write the performance measurements to.
    /// If this is blank, we write to stdout.
    /// Only node 0 writes status output.
    std::string ofile_;

  public:
    /// Max number of requests.  Defaults to 1000.
    int max_req_num;
    /// Message alignment (not really used for anything yet).  Defaults to 64.
    int msg_alignment;
    /// Max message size.  Defaults to 1<<22.
    int max_msg_size;
    /// Number of times through the loop.  Defaults to 100.
    int loop;
    /// Number of iterations skipped before timing starts.  Defaults to 10.
    int skip;
    /// Number of times through the loop for large messages.  Defaults to 20.
    int loop_large;
    /// Number of skipped iterations for large transmissions.  Defaults to 2.
    int skip_large;
    /// What is considered a large message? Defaults to 8192.
    int large_message_size;
    
  protected:
    /// Hi.
    osulat(const std::string &ofile);

  public:
    /// It's creation time.
    static boost::shared_ptr<osulat> construct(const std::string &ofile = "");

    /// Goodbye.
    virtual ~osulat() throw();

    /// Get a copy.
    boost::shared_ptr<mpiapp> clone();

    /// Go.
    void run();
  };

} // end of namespace sstmac.

#endif // ! SSTMAC_SKELETON_OSUBENCHMARKS_OSULAT_H
