AC_DEFUN([DUMPI_TEST_MPI],
  [
    dnl Test whether mpi.h is found
    AC_LANG_PUSH(C)
    AC_MSG_CHECKING([whether mpi.h is found])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
          #include <stdlib.h>
        ], [
          int *ac; char ***av;
          MPI_Init(ac, av);
          MPI_Finalize();
          return EXIT_SUCCESS;
        ]
      )],
      AC_MSG_RESULT([yes]),
      AC_MSG_FAILURE([no])
    )

    dnl Is the MPI_Win time recognized?
    AH_TEMPLATE([DUMPI_HAVE_MPI_WIN], [if MPI_Win is a recognized type])
    AC_MSG_CHECKING([whether MPI_Win is a valid type])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *argc;
          char ***argv;
	  MPI_Win mpiwin;
          MPI_Init(argc, argv);
          PMPI_Init(argc, argv);
          MPI_Finalize();
        ])],
      [
        AC_DEFINE(DUMPI_HAVE_MPI_WIN, [1])
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
      ]
    )
  
    AH_TEMPLATE([DUMPI_SKIP_MPI_INIT], [if MPI_Init is missing])
    AC_MSG_CHECKING([whether MPI_Init is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *argc;
          char ***argv;
          MPI_Init(argc, argv);
          PMPI_Init(argc, argv);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
	have_init=yes
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INIT, [1])
        have_init=no
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INIT_THREAD], [if MPI_Init_thread is missing])
    AC_MSG_CHECKING([whether MPI_Init_thread is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *argc;
          char ***argv;
          int required;
          int *provided;
          MPI_Init_thread(argc, argv, required, provided);
          PMPI_Init_thread(argc, argv, required, provided);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
	if test "$have_init" = "no"; then
	  AC_MSG_FAILURE([MPI_Init and MPI_Init_thread both missing -- goodbye])
        else
          AC_MSG_RESULT([no])
          AC_DEFINE(DUMPI_SKIP_MPI_INIT_THREAD, [1])
	fi
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FINALIZE], [if MPI_Finalize is missing])
    AC_MSG_CHECKING([whether MPI_Finalize is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Finalize();
          PMPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_FAILURE([MPI_Finalize not found -- no need to keep going])
        # AC_MSG_RESULT([no])
        # AC_DEFINE(DUMPI_SKIP_MPI_FINALIZE, [1])
      ]
    )

    dnl Test whether MPIO_Request is a separate type or not.
    AH_TEMPLATE([DUMPI_HAVE_MPIO_REQUEST_NULL], [MPIO_REQUEST_NULL defined])
    AC_MSG_CHECKING([whether MPIO_REQUEST_NULL is defined])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPIO_Request req = MPIO_REQUEST_NULL;
        ])],
      [
        AC_MSG_RESULT([yes])
        AC_DEFINE(DUMPI_HAVE_MPIO_REQUEST_NULL, [1])
      ],
      AC_MSG_RESULT([no])
    )

    dnl Test MPI functions

    AH_TEMPLATE([DUMPI_SKIP_MPI_SEND], [if MPI_Send is missing])
    AC_MSG_CHECKING([whether MPI_Send is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int tag;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Send(buf, count, datatype, dest, tag, comm);
          PMPI_Send(buf, count, datatype, dest, tag, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_SEND, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_RECV], [if MPI_Recv is missing])
    AC_MSG_CHECKING([whether MPI_Recv is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int source;
          int tag;
          MPI_Status *status;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Recv(buf, count, datatype, source, tag, comm, status);
          PMPI_Recv(buf, count, datatype, source, tag, comm, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_RECV, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GET_COUNT], [if MPI_Get_count is missing])
    AC_MSG_CHECKING([whether MPI_Get_count is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Status *status;
          MPI_Datatype datatype;
          int *count;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Get_count(status, datatype, count);
          PMPI_Get_count(status, datatype, count);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GET_COUNT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_BSEND], [if MPI_Bsend is missing])
    AC_MSG_CHECKING([whether MPI_Bsend is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int tag;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Bsend(buf, count, datatype, dest, tag, comm);
          PMPI_Bsend(buf, count, datatype, dest, tag, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_BSEND, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_SSEND], [if MPI_Ssend is missing])
    AC_MSG_CHECKING([whether MPI_Ssend is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int tag;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Ssend(buf, count, datatype, dest, tag, comm);
          PMPI_Ssend(buf, count, datatype, dest, tag, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_SSEND, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_RSEND], [if MPI_Rsend is missing])
    AC_MSG_CHECKING([whether MPI_Rsend is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int tag;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Rsend(buf, count, datatype, dest, tag, comm);
          PMPI_Rsend(buf, count, datatype, dest, tag, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_RSEND, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_BUFFER_ATTACH], [if MPI_Buffer_attach is missing])
    AC_MSG_CHECKING([whether MPI_Buffer_attach is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int size;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Buffer_attach(buf, size);
          PMPI_Buffer_attach(buf, size);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_BUFFER_ATTACH, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_BUFFER_DETACH], [if MPI_Buffer_detach is missing])
    AC_MSG_CHECKING([whether MPI_Buffer_detach is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int *size;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Buffer_detach(buf, size);
          PMPI_Buffer_detach(buf, size);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_BUFFER_DETACH, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ISEND], [if MPI_Isend is missing])
    AC_MSG_CHECKING([whether MPI_Isend is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int tag;
          MPI_Comm comm;
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Isend(buf, count, datatype, dest, tag, comm, request);
          PMPI_Isend(buf, count, datatype, dest, tag, comm, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ISEND, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_IBSEND], [if MPI_Ibsend is missing])
    AC_MSG_CHECKING([whether MPI_Ibsend is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int tag;
          MPI_Comm comm;
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Ibsend(buf, count, datatype, dest, tag, comm, request);
          PMPI_Ibsend(buf, count, datatype, dest, tag, comm, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_IBSEND, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ISSEND], [if MPI_Issend is missing])
    AC_MSG_CHECKING([whether MPI_Issend is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int tag;
          MPI_Comm comm;
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Issend(buf, count, datatype, dest, tag, comm, request);
          PMPI_Issend(buf, count, datatype, dest, tag, comm, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ISSEND, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_IRSEND], [if MPI_Irsend is missing])
    AC_MSG_CHECKING([whether MPI_Irsend is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int tag;
          MPI_Comm comm;
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Irsend(buf, count, datatype, dest, tag, comm, request);
          PMPI_Irsend(buf, count, datatype, dest, tag, comm, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_IRSEND, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_IRECV], [if MPI_Irecv is missing])
    AC_MSG_CHECKING([whether MPI_Irecv is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int source;
          int tag;
          MPI_Comm comm;
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Irecv(buf, count, datatype, source, tag, comm, request);
          PMPI_Irecv(buf, count, datatype, source, tag, comm, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_IRECV, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WAIT], [if MPI_Wait is missing])
    AC_MSG_CHECKING([whether MPI_Wait is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Request *request;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Wait(request, status);
          PMPI_Wait(request, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WAIT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TEST], [if MPI_Test is missing])
    AC_MSG_CHECKING([whether MPI_Test is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Request *request;
          int *flag;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Test(request, flag, status);
          PMPI_Test(request, flag, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TEST, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_REQUEST_FREE], [if MPI_Request_free is missing])
    AC_MSG_CHECKING([whether MPI_Request_free is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Request_free(request);
          PMPI_Request_free(request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_REQUEST_FREE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WAITANY], [if MPI_Waitany is missing])
    AC_MSG_CHECKING([whether MPI_Waitany is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPI_Request *requests;
          int *index;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Waitany(count, requests, index, status);
          PMPI_Waitany(count, requests, index, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WAITANY, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TESTANY], [if MPI_Testany is missing])
    AC_MSG_CHECKING([whether MPI_Testany is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPI_Request *requests;
          int *index;
          int *flag;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Testany(count, requests, index, flag, status);
          PMPI_Testany(count, requests, index, flag, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TESTANY, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WAITALL], [if MPI_Waitall is missing])
    AC_MSG_CHECKING([whether MPI_Waitall is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPI_Request *requests;
          MPI_Status *statuses;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Waitall(count, requests, statuses);
          PMPI_Waitall(count, requests, statuses);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WAITALL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TESTALL], [if MPI_Testall is missing])
    AC_MSG_CHECKING([whether MPI_Testall is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPI_Request *requests;
          int *flag;
          MPI_Status *statuses;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Testall(count, requests, flag, statuses);
          PMPI_Testall(count, requests, flag, statuses);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TESTALL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WAITSOME], [if MPI_Waitsome is missing])
    AC_MSG_CHECKING([whether MPI_Waitsome is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPI_Request *requests;
          int *outcount;
          int *indices;
          MPI_Status *statuses;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Waitsome(count, requests, outcount, indices, statuses);
          PMPI_Waitsome(count, requests, outcount, indices, statuses);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WAITSOME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TESTSOME], [if MPI_Testsome is missing])
    AC_MSG_CHECKING([whether MPI_Testsome is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPI_Request *requests;
          int *outcount;
          int *indices;
          MPI_Status *statuses;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Testsome(count, requests, outcount, indices, statuses);
          PMPI_Testsome(count, requests, outcount, indices, statuses);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TESTSOME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_IPROBE], [if MPI_Iprobe is missing])
    AC_MSG_CHECKING([whether MPI_Iprobe is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int source;
          int tag;
          MPI_Comm comm;
          int *flag;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Iprobe(source, tag, comm, flag, status);
          PMPI_Iprobe(source, tag, comm, flag, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_IPROBE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_PROBE], [if MPI_Probe is missing])
    AC_MSG_CHECKING([whether MPI_Probe is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int source;
          int tag;
          MPI_Comm comm;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Probe(source, tag, comm, status);
          PMPI_Probe(source, tag, comm, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_PROBE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_CANCEL], [if MPI_Cancel is missing])
    AC_MSG_CHECKING([whether MPI_Cancel is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Cancel(request);
          PMPI_Cancel(request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_CANCEL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TEST_CANCELLED], [if MPI_Test_cancelled is missing])
    AC_MSG_CHECKING([whether MPI_Test_cancelled is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Status *status;
          int *cancelled;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Test_cancelled(status, cancelled);
          PMPI_Test_cancelled(status, cancelled);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TEST_CANCELLED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_SEND_INIT], [if MPI_Send_init is missing])
    AC_MSG_CHECKING([whether MPI_Send_init is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int tag;
          MPI_Comm comm;
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Send_init(buf, count, datatype, dest, tag, comm, request);
          PMPI_Send_init(buf, count, datatype, dest, tag, comm, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_SEND_INIT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_BSEND_INIT], [if MPI_Bsend_init is missing])
    AC_MSG_CHECKING([whether MPI_Bsend_init is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int tag;
          MPI_Comm comm;
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Bsend_init(buf, count, datatype, dest, tag, comm, request);
          PMPI_Bsend_init(buf, count, datatype, dest, tag, comm, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_BSEND_INIT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_SSEND_INIT], [if MPI_Ssend_init is missing])
    AC_MSG_CHECKING([whether MPI_Ssend_init is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int tag;
          MPI_Comm comm;
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Ssend_init(buf, count, datatype, dest, tag, comm, request);
          PMPI_Ssend_init(buf, count, datatype, dest, tag, comm, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_SSEND_INIT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_RSEND_INIT], [if MPI_Rsend_init is missing])
    AC_MSG_CHECKING([whether MPI_Rsend_init is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int tag;
          MPI_Comm comm;
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Rsend_init(buf, count, datatype, dest, tag, comm, request);
          PMPI_Rsend_init(buf, count, datatype, dest, tag, comm, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_RSEND_INIT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_RECV_INIT], [if MPI_Recv_init is missing])
    AC_MSG_CHECKING([whether MPI_Recv_init is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int source;
          int tag;
          MPI_Comm comm;
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Recv_init(buf, count, datatype, source, tag, comm, request);
          PMPI_Recv_init(buf, count, datatype, source, tag, comm, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_RECV_INIT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_START], [if MPI_Start is missing])
    AC_MSG_CHECKING([whether MPI_Start is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Start(request);
          PMPI_Start(request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_START, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_STARTALL], [if MPI_Startall is missing])
    AC_MSG_CHECKING([whether MPI_Startall is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Startall(count, request);
          PMPI_Startall(count, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_STARTALL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_SENDRECV], [if MPI_Sendrecv is missing])
    AC_MSG_CHECKING([whether MPI_Sendrecv is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          int sendcount;
          MPI_Datatype sendtype;
          int dest;
          int sendtag;
          void *recvbuf;
          int recvcount;
          MPI_Datatype recvtype;
          int source;
          int recvtag;
          MPI_Comm comm;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status);
          PMPI_Sendrecv(sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_SENDRECV, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_SENDRECV_REPLACE], [if MPI_Sendrecv_replace is missing])
    AC_MSG_CHECKING([whether MPI_Sendrecv_replace is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int count;
          MPI_Datatype datatype;
          int dest;
          int sendtag;
          int source;
          int recvtag;
          MPI_Comm comm;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Sendrecv_replace(buf, count, datatype, dest, sendtag, source, recvtag, comm, status);
          PMPI_Sendrecv_replace(buf, count, datatype, dest, sendtag, source, recvtag, comm, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_SENDRECV_REPLACE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_CONTIGUOUS], [if MPI_Type_contiguous is missing])
    AC_MSG_CHECKING([whether MPI_Type_contiguous is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPI_Datatype oldtype;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_contiguous(count, oldtype, newtype);
          PMPI_Type_contiguous(count, oldtype, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_CONTIGUOUS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_VECTOR], [if MPI_Type_vector is missing])
    AC_MSG_CHECKING([whether MPI_Type_vector is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          int blocklength;
          int stride;
          MPI_Datatype oldtype;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_vector(count, blocklength, stride, oldtype, newtype);
          PMPI_Type_vector(count, blocklength, stride, oldtype, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_VECTOR, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_HVECTOR], [if MPI_Type_hvector is missing])
    AC_MSG_CHECKING([whether MPI_Type_hvector is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          int blocklength;
          MPI_Aint stride;
          MPI_Datatype oldtype;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_hvector(count, blocklength, stride, oldtype, newtype);
          PMPI_Type_hvector(count, blocklength, stride, oldtype, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_HVECTOR, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_INDEXED], [if MPI_Type_indexed is missing])
    AC_MSG_CHECKING([whether MPI_Type_indexed is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          int *lengths;
          int *indices;
          MPI_Datatype oldtype;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_indexed(count, lengths, indices, oldtype, newtype);
          PMPI_Type_indexed(count, lengths, indices, oldtype, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_INDEXED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_HINDEXED], [if MPI_Type_hindexed is missing])
    AC_MSG_CHECKING([whether MPI_Type_hindexed is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          int *lengths;
          MPI_Aint *indices;
          MPI_Datatype oldtype;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_hindexed(count, lengths, indices, oldtype, newtype);
          PMPI_Type_hindexed(count, lengths, indices, oldtype, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_HINDEXED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_STRUCT], [if MPI_Type_struct is missing])
    AC_MSG_CHECKING([whether MPI_Type_struct is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          int *lengths;
          MPI_Aint *indices;
          MPI_Datatype *oldtypes;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_struct(count, lengths, indices, oldtypes, newtype);
          PMPI_Type_struct(count, lengths, indices, oldtypes, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_STRUCT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ADDRESS], [if MPI_Address is missing])
    AC_MSG_CHECKING([whether MPI_Address is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          MPI_Aint *address;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Address(buf, address);
          PMPI_Address(buf, address);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ADDRESS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_EXTENT], [if MPI_Type_extent is missing])
    AC_MSG_CHECKING([whether MPI_Type_extent is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          MPI_Aint *extent;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_extent(datatype, extent);
          PMPI_Type_extent(datatype, extent);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_EXTENT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_SIZE], [if MPI_Type_size is missing])
    AC_MSG_CHECKING([whether MPI_Type_size is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          int *size;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_size(datatype, size);
          PMPI_Type_size(datatype, size);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_SIZE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_LB], [if MPI_Type_lb is missing])
    AC_MSG_CHECKING([whether MPI_Type_lb is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          MPI_Aint *lb;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_lb(datatype, lb);
          PMPI_Type_lb(datatype, lb);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_LB, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_UB], [if MPI_Type_ub is missing])
    AC_MSG_CHECKING([whether MPI_Type_ub is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          MPI_Aint *ub;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_ub(datatype, ub);
          PMPI_Type_ub(datatype, ub);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_UB, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_COMMIT], [if MPI_Type_commit is missing])
    AC_MSG_CHECKING([whether MPI_Type_commit is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype *datatype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_commit(datatype);
          PMPI_Type_commit(datatype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_COMMIT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_FREE], [if MPI_Type_free is missing])
    AC_MSG_CHECKING([whether MPI_Type_free is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype *datatype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_free(datatype);
          PMPI_Type_free(datatype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_FREE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GET_ELEMENTS], [if MPI_Get_elements is missing])
    AC_MSG_CHECKING([whether MPI_Get_elements is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Status *status;
          MPI_Datatype datatype;
          int *elements;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Get_elements(status, datatype, elements);
          PMPI_Get_elements(status, datatype, elements);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GET_ELEMENTS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_PACK], [if MPI_Pack is missing])
    AC_MSG_CHECKING([whether MPI_Pack is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *inbuf;
          int incount;
          MPI_Datatype datatype;
          void *outbuf;
          int outcount;
          int *position;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm comm;
          MPI_Pack(inbuf, incount, datatype, outbuf, outcount, position, comm);
          PMPI_Pack(inbuf, incount, datatype, outbuf, outcount, position, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_PACK, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_UNPACK], [if MPI_Unpack is missing])
    AC_MSG_CHECKING([whether MPI_Unpack is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *inbuf;
          int incount;
          int *position;
          void *outbuf;
          int outcount;
          MPI_Datatype datatype;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Unpack(inbuf, incount, position, outbuf, outcount, datatype, comm);
          PMPI_Unpack(inbuf, incount, position, outbuf, outcount, datatype, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_UNPACK, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_PACK_SIZE], [if MPI_Pack_size is missing])
    AC_MSG_CHECKING([whether MPI_Pack_size is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int incount;
          MPI_Datatype datatype;
          MPI_Comm comm;
          int *size;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Pack_size(incount, datatype, comm, size);
          PMPI_Pack_size(incount, datatype, comm, size);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_PACK_SIZE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_BARRIER], [if MPI_Barrier is missing])
    AC_MSG_CHECKING([whether MPI_Barrier is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Barrier(comm);
          PMPI_Barrier(comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_BARRIER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_BCAST], [if MPI_Bcast is missing])
    AC_MSG_CHECKING([whether MPI_Bcast is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          int count;
          MPI_Datatype datatype;
          int root;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Bcast(sendbuf, count, datatype, root, comm);
          PMPI_Bcast(sendbuf, count, datatype, root, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_BCAST, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GATHER], [if MPI_Gather is missing])
    AC_MSG_CHECKING([whether MPI_Gather is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *inbuf;
          int sendcount;
          MPI_Datatype sendtype;
          void *recvbuf;
          int recvcount;
          MPI_Datatype recvtype;
          int root;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Gather(inbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
          PMPI_Gather(inbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GATHER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GATHERV], [if MPI_Gatherv is missing])
    AC_MSG_CHECKING([whether MPI_Gatherv is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          int sendcount;
          MPI_Datatype sendtype;
          void *recvbuf;
          int *recvcounts;
          int *displs;
          MPI_Datatype recvtype;
          int root;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Gatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
          PMPI_Gatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, root, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GATHERV, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_SCATTER], [if MPI_Scatter is missing])
    AC_MSG_CHECKING([whether MPI_Scatter is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          int sendcount;
          MPI_Datatype sendtype;
          void *recvbuf;
          int recvcount;
          MPI_Datatype recvtype;
          int root;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
          PMPI_Scatter(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, root, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_SCATTER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_SCATTERV], [if MPI_Scatterv is missing])
    AC_MSG_CHECKING([whether MPI_Scatterv is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          int *sendcounts;
          int *displs;
          MPI_Datatype sendtype;
          void *recvbuf;
          int recvcount;
          MPI_Datatype recvtype;
          int root;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Scatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
          PMPI_Scatterv(sendbuf, sendcounts, displs, sendtype, recvbuf, recvcount, recvtype, root, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_SCATTERV, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ALLGATHER], [if MPI_Allgather is missing])
    AC_MSG_CHECKING([whether MPI_Allgather is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          int sendcount;
          MPI_Datatype sendtype;
          void *recvbuf;
          int recvcount;
          MPI_Datatype recvtype;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
          PMPI_Allgather(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ALLGATHER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ALLGATHERV], [if MPI_Allgatherv is missing])
    AC_MSG_CHECKING([whether MPI_Allgatherv is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          int sendcount;
          MPI_Datatype sendtype;
          void *recvbuf;
          int *recvcounts;
          int *displs;
          MPI_Datatype recvtype;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
          PMPI_Allgatherv(sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ALLGATHERV, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ALLTOALL], [if MPI_Alltoall is missing])
    AC_MSG_CHECKING([whether MPI_Alltoall is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          int sendcount;
          MPI_Datatype sendtype;
          void *recvbuf;
          int recvcount;
          MPI_Datatype recvtype;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
          PMPI_Alltoall(sendbuf, sendcount, sendtype, recvbuf, recvcount, recvtype, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ALLTOALL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ALLTOALLV], [if MPI_Alltoallv is missing])
    AC_MSG_CHECKING([whether MPI_Alltoallv is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          int *sendcounts;
          int *senddispls;
          MPI_Datatype sendtype;
          void *recvbuf;
          int *recvcounts;
          int *recvdispls;
          MPI_Datatype recvtype;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Alltoallv(sendbuf, sendcounts, senddispls, sendtype, recvbuf, recvcounts, recvdispls, recvtype, comm);
          PMPI_Alltoallv(sendbuf, sendcounts, senddispls, sendtype, recvbuf, recvcounts, recvdispls, recvtype, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ALLTOALLV, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_REDUCE], [if MPI_Reduce is missing])
    AC_MSG_CHECKING([whether MPI_Reduce is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          void *recvbuf;
          int count;
          MPI_Datatype datatype;
          MPI_Op op;
          int root;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Reduce(sendbuf, recvbuf, count, datatype, op, root, comm);
          PMPI_Reduce(sendbuf, recvbuf, count, datatype, op, root, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_REDUCE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_OP_CREATE], [if MPI_Op_create is missing])
    AC_MSG_CHECKING([whether MPI_Op_create is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_User_function *fptr;
          int commute;
          MPI_Op *op;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Op_create(fptr, commute, op);
          PMPI_Op_create(fptr, commute, op);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_OP_CREATE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_OP_FREE], [if MPI_Op_free is missing])
    AC_MSG_CHECKING([whether MPI_Op_free is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Op *op;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Op_free(op);
          PMPI_Op_free(op);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_OP_FREE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ALLREDUCE], [if MPI_Allreduce is missing])
    AC_MSG_CHECKING([whether MPI_Allreduce is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          void *recvbuf;
          int count;
          MPI_Datatype datatype;
          MPI_Op op;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm);
          PMPI_Allreduce(sendbuf, recvbuf, count, datatype, op, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ALLREDUCE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_REDUCE_SCATTER], [if MPI_Reduce_scatter is missing])
    AC_MSG_CHECKING([whether MPI_Reduce_scatter is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          void *recvbuf;
          int *recvcounts;
          MPI_Datatype datatype;
          MPI_Op op;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Reduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm);
          PMPI_Reduce_scatter(sendbuf, recvbuf, recvcounts, datatype, op, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_REDUCE_SCATTER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_SCAN], [if MPI_Scan is missing])
    AC_MSG_CHECKING([whether MPI_Scan is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          void *recvbuf;
          int count;
          MPI_Datatype datatype;
          MPI_Op op;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
          PMPI_Scan(sendbuf, recvbuf, count, datatype, op, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_SCAN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GROUP_SIZE], [if MPI_Group_size is missing])
    AC_MSG_CHECKING([whether MPI_Group_size is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Group group;
          int *size;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Group_size(group, size);
          PMPI_Group_size(group, size);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GROUP_SIZE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GROUP_RANK], [if MPI_Group_rank is missing])
    AC_MSG_CHECKING([whether MPI_Group_rank is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Group group;
          int *rank;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Group_rank(group, rank);
          PMPI_Group_rank(group, rank);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GROUP_RANK, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GROUP_TRANSLATE_RANKS], [if MPI_Group_translate_ranks is missing])
    AC_MSG_CHECKING([whether MPI_Group_translate_ranks is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Group group1;
          int count;
          int *ranks1;
          MPI_Group group2;
          int *ranks2;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Group_translate_ranks(group1, count, ranks1, group2, ranks2);
          PMPI_Group_translate_ranks(group1, count, ranks1, group2, ranks2);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GROUP_TRANSLATE_RANKS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GROUP_COMPARE], [if MPI_Group_compare is missing])
    AC_MSG_CHECKING([whether MPI_Group_compare is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Group group1;
          MPI_Group group2;
          int *result;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Group_compare(group1, group2, result);
          PMPI_Group_compare(group1, group2, result);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GROUP_COMPARE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_GROUP], [if MPI_Comm_group is missing])
    AC_MSG_CHECKING([whether MPI_Comm_group is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          MPI_Group *group;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_group(comm, group);
          PMPI_Comm_group(comm, group);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_GROUP, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GROUP_UNION], [if MPI_Group_union is missing])
    AC_MSG_CHECKING([whether MPI_Group_union is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Group group1;
          MPI_Group group2;
          MPI_Group *newgroup;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Group_union(group1, group2, newgroup);
          PMPI_Group_union(group1, group2, newgroup);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GROUP_UNION, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GROUP_INTERSECTION], [if MPI_Group_intersection is missing])
    AC_MSG_CHECKING([whether MPI_Group_intersection is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Group group1;
          MPI_Group group2;
          MPI_Group *newgroup;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Group_intersection(group1, group2, newgroup);
          PMPI_Group_intersection(group1, group2, newgroup);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GROUP_INTERSECTION, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GROUP_DIFFERENCE], [if MPI_Group_difference is missing])
    AC_MSG_CHECKING([whether MPI_Group_difference is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Group group1;
          MPI_Group group2;
          MPI_Group *newgroup;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Group_difference(group1, group2, newgroup);
          PMPI_Group_difference(group1, group2, newgroup);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GROUP_DIFFERENCE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GROUP_INCL], [if MPI_Group_incl is missing])
    AC_MSG_CHECKING([whether MPI_Group_incl is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Group group;
          int count;
          int *ranks;
          MPI_Group *newgroup;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Group_incl(group, count, ranks, newgroup);
          PMPI_Group_incl(group, count, ranks, newgroup);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GROUP_INCL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GROUP_EXCL], [if MPI_Group_excl is missing])
    AC_MSG_CHECKING([whether MPI_Group_excl is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Group group;
          int count;
          int *ranks;
          MPI_Group *newgroup;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Group_excl(group, count, ranks, newgroup);
          PMPI_Group_excl(group, count, ranks, newgroup);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GROUP_EXCL, [1])
      ]
    )

    dnl Note that the source block needs to be double quoted ([[...]]) in order
    dnl to get autoconf to stop stripping away the array brackets (opinion withheld).
    AH_TEMPLATE([DUMPI_SKIP_MPI_GROUP_RANGE_INCL], [if MPI_Group_range_incl is missing])
    AC_MSG_CHECKING([whether MPI_Group_range_incl is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [[
          MPI_Group group;
          int count;
          int ranges[10][3];
          MPI_Group *newgroup;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Group_range_incl(group, count, ranges, newgroup);
          PMPI_Group_range_incl(group, count, ranges, newgroup);
          MPI_Finalize();
        ]])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GROUP_RANGE_INCL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GROUP_RANGE_EXCL], [if MPI_Group_range_excl is missing])
    AC_MSG_CHECKING([whether MPI_Group_range_excl is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [[
          MPI_Group group;
          int count;
          int ranges[10][3];
          MPI_Group *newgroup;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Group_range_excl(group, count, ranges, newgroup);
          PMPI_Group_range_excl(group, count, ranges, newgroup);
          MPI_Finalize();
        ]])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GROUP_RANGE_EXCL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GROUP_FREE], [if MPI_Group_free is missing])
    AC_MSG_CHECKING([whether MPI_Group_free is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Group *group;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Group_free(group);
          PMPI_Group_free(group);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GROUP_FREE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_SIZE], [if MPI_Comm_size is missing])
    AC_MSG_CHECKING([whether MPI_Comm_size is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int *size;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_size(comm, size);
          PMPI_Comm_size(comm, size);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_SIZE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_RANK], [if MPI_Comm_rank is missing])
    AC_MSG_CHECKING([whether MPI_Comm_rank is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int *rank;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_rank(comm, rank);
          PMPI_Comm_rank(comm, rank);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_RANK, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_COMPARE], [if MPI_Comm_compare is missing])
    AC_MSG_CHECKING([whether MPI_Comm_compare is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm1;
          MPI_Comm comm2;
          int *result;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_compare(comm1, comm2, result);
          PMPI_Comm_compare(comm1, comm2, result);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_COMPARE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_DUP], [if MPI_Comm_dup is missing])
    AC_MSG_CHECKING([whether MPI_Comm_dup is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm oldcomm;
          MPI_Comm *newcomm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_dup(oldcomm, newcomm);
          PMPI_Comm_dup(oldcomm, newcomm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_DUP, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_CREATE], [if MPI_Comm_create is missing])
    AC_MSG_CHECKING([whether MPI_Comm_create is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm oldcomm;
          MPI_Group group;
          MPI_Comm *newcomm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_create(oldcomm, group, newcomm);
          PMPI_Comm_create(oldcomm, group, newcomm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_CREATE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_SPLIT], [if MPI_Comm_split is missing])
    AC_MSG_CHECKING([whether MPI_Comm_split is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm oldcomm;
          int color;
          int key;
          MPI_Comm *newcomm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_split(oldcomm, color, key, newcomm);
          PMPI_Comm_split(oldcomm, color, key, newcomm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_SPLIT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_FREE], [if MPI_Comm_free is missing])
    AC_MSG_CHECKING([whether MPI_Comm_free is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm *comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_free(comm);
          PMPI_Comm_free(comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_FREE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_TEST_INTER], [if MPI_Comm_test_inter is missing])
    AC_MSG_CHECKING([whether MPI_Comm_test_inter is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int *inter;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_test_inter(comm, inter);
          PMPI_Comm_test_inter(comm, inter);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_TEST_INTER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_REMOTE_SIZE], [if MPI_Comm_remote_size is missing])
    AC_MSG_CHECKING([whether MPI_Comm_remote_size is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int *size;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_remote_size(comm, size);
          PMPI_Comm_remote_size(comm, size);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_REMOTE_SIZE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_REMOTE_GROUP], [if MPI_Comm_remote_group is missing])
    AC_MSG_CHECKING([whether MPI_Comm_remote_group is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          MPI_Group *group;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_remote_group(comm, group);
          PMPI_Comm_remote_group(comm, group);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_REMOTE_GROUP, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INTERCOMM_CREATE], [if MPI_Intercomm_create is missing])
    AC_MSG_CHECKING([whether MPI_Intercomm_create is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm localcomm;
          int localleader;
          MPI_Comm remotecomm;
          int remoteleader;
          int tag;
          MPI_Comm *newcomm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Intercomm_create(localcomm, localleader, remotecomm, remoteleader, tag, newcomm);
          PMPI_Intercomm_create(localcomm, localleader, remotecomm, remoteleader, tag, newcomm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INTERCOMM_CREATE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INTERCOMM_MERGE], [if MPI_Intercomm_merge is missing])
    AC_MSG_CHECKING([whether MPI_Intercomm_merge is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int high;
          MPI_Comm *newcomm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Intercomm_merge(comm, high, newcomm);
          PMPI_Intercomm_merge(comm, high, newcomm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INTERCOMM_MERGE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_KEYVAL_CREATE], [if MPI_Keyval_create is missing])
    AC_MSG_CHECKING([whether MPI_Keyval_create is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Copy_function *copyfunc;
          MPI_Delete_function *delfunc;
          int *key;
          void *state;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Keyval_create(copyfunc, delfunc, key, state);
          PMPI_Keyval_create(copyfunc, delfunc, key, state);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_KEYVAL_CREATE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_KEYVAL_FREE], [if MPI_Keyval_free is missing])
    AC_MSG_CHECKING([whether MPI_Keyval_free is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *key;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Keyval_free(key);
          PMPI_Keyval_free(key);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_KEYVAL_FREE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ATTR_PUT], [if MPI_Attr_put is missing])
    AC_MSG_CHECKING([whether MPI_Attr_put is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int key;
          void *attr;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Attr_put(comm, key, attr);
          PMPI_Attr_put(comm, key, attr);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ATTR_PUT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ATTR_GET], [if MPI_Attr_get is missing])
    AC_MSG_CHECKING([whether MPI_Attr_get is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int key;
          void *attr;
          int *flag;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Attr_get(comm, key, attr, flag);
          PMPI_Attr_get(comm, key, attr, flag);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ATTR_GET, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ATTR_DELETE], [if MPI_Attr_delete is missing])
    AC_MSG_CHECKING([whether MPI_Attr_delete is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int key;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Attr_delete(comm, key);
          PMPI_Attr_delete(comm, key);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ATTR_DELETE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TOPO_TEST], [if MPI_Topo_test is missing])
    AC_MSG_CHECKING([whether MPI_Topo_test is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int *topo;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Topo_test(comm, topo);
          PMPI_Topo_test(comm, topo);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TOPO_TEST, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_CART_CREATE], [if MPI_Cart_create is missing])
    AC_MSG_CHECKING([whether MPI_Cart_create is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm oldcomm;
          int ndim;
          int *dims;
          int *periods;
          int reorder;
          MPI_Comm *newcomm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Cart_create(oldcomm, ndim, dims, periods, reorder, newcomm);
          PMPI_Cart_create(oldcomm, ndim, dims, periods, reorder, newcomm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_CART_CREATE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_DIMS_CREATE], [if MPI_Dims_create is missing])
    AC_MSG_CHECKING([whether MPI_Dims_create is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int nodes;
          int ndim;
          int *dims;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Dims_create(nodes, ndim, dims);
          PMPI_Dims_create(nodes, ndim, dims);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_DIMS_CREATE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GRAPH_CREATE], [if MPI_Graph_create is missing])
    AC_MSG_CHECKING([whether MPI_Graph_create is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm oldcomm;
          int nodes;
          int *index;
          int *edges;
          int reorder;
          MPI_Comm *newcomm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Graph_create(oldcomm, nodes, index, edges, reorder, newcomm);
          PMPI_Graph_create(oldcomm, nodes, index, edges, reorder, newcomm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GRAPH_CREATE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GRAPHDIMS_GET], [if MPI_Graphdims_get is missing])
    AC_MSG_CHECKING([whether MPI_Graphdims_get is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int *nodes;
          int *edges;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Graphdims_get(comm, nodes, edges);
          PMPI_Graphdims_get(comm, nodes, edges);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GRAPHDIMS_GET, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GRAPH_GET], [if MPI_Graph_get is missing])
    AC_MSG_CHECKING([whether MPI_Graph_get is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int maxindex;
          int maxedges;
          int *index;
          int *edges;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Graph_get(comm, maxindex, maxedges, index, edges);
          PMPI_Graph_get(comm, maxindex, maxedges, index, edges);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GRAPH_GET, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_CARTDIM_GET], [if MPI_Cartdim_get is missing])
    AC_MSG_CHECKING([whether MPI_Cartdim_get is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int *ndim;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Cartdim_get(comm, ndim);
          PMPI_Cartdim_get(comm, ndim);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_CARTDIM_GET, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_CART_GET], [if MPI_Cart_get is missing])
    AC_MSG_CHECKING([whether MPI_Cart_get is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int maxdims;
          int *dims;
          int *periods;
          int *coords;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Cart_get(comm, maxdims, dims, periods, coords);
          PMPI_Cart_get(comm, maxdims, dims, periods, coords);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_CART_GET, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_CART_RANK], [if MPI_Cart_rank is missing])
    AC_MSG_CHECKING([whether MPI_Cart_rank is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int *coords;
          int *rank;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Cart_rank(comm, coords, rank);
          PMPI_Cart_rank(comm, coords, rank);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_CART_RANK, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_CART_COORDS], [if MPI_Cart_coords is missing])
    AC_MSG_CHECKING([whether MPI_Cart_coords is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int rank;
          int maxdims;
          int *coords;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Cart_coords(comm, rank, maxdims, coords);
          PMPI_Cart_coords(comm, rank, maxdims, coords);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_CART_COORDS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GRAPH_NEIGHBORS_COUNT], [if MPI_Graph_neighbors_count is missing])
    AC_MSG_CHECKING([whether MPI_Graph_neighbors_count is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int rank;
          int *nneigh;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Graph_neighbors_count(comm, rank, nneigh);
          PMPI_Graph_neighbors_count(comm, rank, nneigh);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GRAPH_NEIGHBORS_COUNT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GRAPH_NEIGHBORS], [if MPI_Graph_neighbors is missing])
    AC_MSG_CHECKING([whether MPI_Graph_neighbors is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int rank;
          int maxneighbors;
          int *neighbors;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Graph_neighbors(comm, rank, maxneighbors, neighbors);
          PMPI_Graph_neighbors(comm, rank, maxneighbors, neighbors);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GRAPH_NEIGHBORS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_CART_SHIFT], [if MPI_Cart_shift is missing])
    AC_MSG_CHECKING([whether MPI_Cart_shift is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int direction;
          int displ;
          int *source;
          int *dest;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Cart_shift(comm, direction, displ, source, dest);
          PMPI_Cart_shift(comm, direction, displ, source, dest);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_CART_SHIFT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_CART_SUB], [if MPI_Cart_sub is missing])
    AC_MSG_CHECKING([whether MPI_Cart_sub is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm oldcomm;
          int *remain_dims;
          MPI_Comm *newcomm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Cart_sub(oldcomm, remain_dims, newcomm);
          PMPI_Cart_sub(oldcomm, remain_dims, newcomm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_CART_SUB, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_CART_MAP], [if MPI_Cart_map is missing])
    AC_MSG_CHECKING([whether MPI_Cart_map is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int ndim;
          int *dims;
          int *period;
          int *newrank;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Cart_map(comm, ndim, dims, period, newrank);
          PMPI_Cart_map(comm, ndim, dims, period, newrank);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_CART_MAP, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GRAPH_MAP], [if MPI_Graph_map is missing])
    AC_MSG_CHECKING([whether MPI_Graph_map is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int nodes;
          int *index;
          int *edges;
          int *newrank;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Graph_map(comm, nodes, index, edges, newrank);
          PMPI_Graph_map(comm, nodes, index, edges, newrank);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GRAPH_MAP, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GET_PROCESSOR_NAME], [if MPI_Get_processor_name is missing])
    AC_MSG_CHECKING([whether MPI_Get_processor_name is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *name;
          int *resultlen;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Get_processor_name(name, resultlen);
          PMPI_Get_processor_name(name, resultlen);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GET_PROCESSOR_NAME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GET_VERSION], [if MPI_Get_version is missing])
    AC_MSG_CHECKING([whether MPI_Get_version is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *version;
          int *subversion;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Get_version(version, subversion);
          PMPI_Get_version(version, subversion);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GET_VERSION, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ERRHANDLER_CREATE], [if MPI_Errhandler_create is missing])
    AC_MSG_CHECKING([whether MPI_Errhandler_create is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Handler_function *function;
          MPI_Errhandler *errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Errhandler_create(function, errhandler);
          PMPI_Errhandler_create(function, errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ERRHANDLER_CREATE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ERRHANDLER_SET], [if MPI_Errhandler_set is missing])
    AC_MSG_CHECKING([whether MPI_Errhandler_set is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          MPI_Errhandler errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Errhandler_set(comm, errhandler);
          PMPI_Errhandler_set(comm, errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ERRHANDLER_SET, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ERRHANDLER_GET], [if MPI_Errhandler_get is missing])
    AC_MSG_CHECKING([whether MPI_Errhandler_get is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          MPI_Errhandler *errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Errhandler_get(comm, errhandler);
          PMPI_Errhandler_get(comm, errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ERRHANDLER_GET, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ERRHANDLER_FREE], [if MPI_Errhandler_free is missing])
    AC_MSG_CHECKING([whether MPI_Errhandler_free is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Errhandler *errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Errhandler_free(errhandler);
          PMPI_Errhandler_free(errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ERRHANDLER_FREE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ERROR_STRING], [if MPI_Error_string is missing])
    AC_MSG_CHECKING([whether MPI_Error_string is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int errorcode;
          char *errorstring;
          int *resultlen;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Error_string(errorcode, errorstring, resultlen);
          PMPI_Error_string(errorcode, errorstring, resultlen);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ERROR_STRING, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ERROR_CLASS], [if MPI_Error_class is missing])
    AC_MSG_CHECKING([whether MPI_Error_class is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int errorcode;
          int *errorclass;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Error_class(errorcode, errorclass);
          PMPI_Error_class(errorcode, errorclass);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ERROR_CLASS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WTIME], [if MPI_Wtime is missing])
    AC_MSG_CHECKING([whether MPI_Wtime is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Wtime();
          PMPI_Wtime();
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WTIME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WTICK], [if MPI_Wtick is missing])
    AC_MSG_CHECKING([whether MPI_Wtick is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Wtick();
          PMPI_Wtick();
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WTICK, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INITIALIZED], [if MPI_Initialized is missing])
    AC_MSG_CHECKING([whether MPI_Initialized is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *result;
          MPI_Initialized(result);
          PMPI_Initialized(result);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INITIALIZED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ABORT], [if MPI_Abort is missing])
    AC_MSG_CHECKING([whether MPI_Abort is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int errorcode;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Abort(comm, errorcode);
          PMPI_Abort(comm, errorcode);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ABORT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_CLOSE_PORT], [if MPI_Close_port is missing])
    AC_MSG_CHECKING([whether MPI_Close_port is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *portname;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Close_port(portname);
          PMPI_Close_port(portname);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_CLOSE_PORT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_ACCEPT], [if MPI_Comm_accept is missing])
    AC_MSG_CHECKING([whether MPI_Comm_accept is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *portname;
          MPI_Info info;
          int root;
          MPI_Comm oldcomm;
          MPI_Comm *newcomm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_accept(portname, info, root, oldcomm, newcomm);
          PMPI_Comm_accept(portname, info, root, oldcomm, newcomm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_ACCEPT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_CONNECT], [if MPI_Comm_connect is missing])
    AC_MSG_CHECKING([whether MPI_Comm_connect is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *portname;
          MPI_Info info;
          int root;
          MPI_Comm oldcomm;
          MPI_Comm *newcomm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_connect(portname, info, root, oldcomm, newcomm);
          PMPI_Comm_connect(portname, info, root, oldcomm, newcomm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_CONNECT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_DISCONNECT], [if MPI_Comm_disconnect is missing])
    AC_MSG_CHECKING([whether MPI_Comm_disconnect is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm *comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_disconnect(comm);
          PMPI_Comm_disconnect(comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_DISCONNECT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_GET_PARENT], [if MPI_Comm_get_parent is missing])
    AC_MSG_CHECKING([whether MPI_Comm_get_parent is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm *parent;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_get_parent(parent);
          PMPI_Comm_get_parent(parent);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_GET_PARENT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_JOIN], [if MPI_Comm_join is missing])
    AC_MSG_CHECKING([whether MPI_Comm_join is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int fd;
          MPI_Comm *comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_join(fd, comm);
          PMPI_Comm_join(fd, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_JOIN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_SPAWN], [if MPI_Comm_spawn is missing])
    AC_MSG_CHECKING([whether MPI_Comm_spawn is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *command;
          char **argv;
          int maxprocs;
          MPI_Info info;
          int root;
          MPI_Comm oldcomm;
          MPI_Comm *newcomm;
          int *errcodes;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_spawn(command, argv, maxprocs, info, root, oldcomm, newcomm, errcodes);
          PMPI_Comm_spawn(command, argv, maxprocs, info, root, oldcomm, newcomm, errcodes);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_SPAWN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_SPAWN_MULTIPLE], [if MPI_Comm_spawn_multiple is missing])
    AC_MSG_CHECKING([whether MPI_Comm_spawn_multiple is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          char **commands;
          char ***argvs;
          int *maxprocs;
          MPI_Info *info;
          int root;
          MPI_Comm oldcomm;
          MPI_Comm *newcomm;
          int *errcodes;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_spawn_multiple(count, commands, argvs, maxprocs, info, root, oldcomm, newcomm, errcodes);
          PMPI_Comm_spawn_multiple(count, commands, argvs, maxprocs, info, root, oldcomm, newcomm, errcodes);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_SPAWN_MULTIPLE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_LOOKUP_NAME], [if MPI_Lookup_name is missing])
    AC_MSG_CHECKING([whether MPI_Lookup_name is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *servicename;
          MPI_Info info;
          char *portname;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Lookup_name(servicename, info, portname);
          PMPI_Lookup_name(servicename, info, portname);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_LOOKUP_NAME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_OPEN_PORT], [if MPI_Open_port is missing])
    AC_MSG_CHECKING([whether MPI_Open_port is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Info info;
          char *portname;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Open_port(info, portname);
          PMPI_Open_port(info, portname);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_OPEN_PORT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_PUBLISH_NAME], [if MPI_Publish_name is missing])
    AC_MSG_CHECKING([whether MPI_Publish_name is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *servicename;
          MPI_Info info;
          char *portname;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Publish_name(servicename, info, portname);
          PMPI_Publish_name(servicename, info, portname);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_PUBLISH_NAME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_UNPUBLISH_NAME], [if MPI_Unpublish_name is missing])
    AC_MSG_CHECKING([whether MPI_Unpublish_name is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *servicename;
          MPI_Info info;
          char *portname;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Unpublish_name(servicename, info, portname);
          PMPI_Unpublish_name(servicename, info, portname);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_UNPUBLISH_NAME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ACCUMULATE], [if MPI_Accumulate is missing])
    AC_MSG_CHECKING([whether MPI_Accumulate is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int origincount;
          MPI_Datatype origintype;
          int targetrank;
          MPI_Aint targetdisp;
          int targetcount;
          MPI_Datatype targettype;
          MPI_Op op;
          MPI_Win win;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Accumulate(buf, origincount, origintype, targetrank, targetdisp, targetcount, targettype, op, win);
          PMPI_Accumulate(buf, origincount, origintype, targetrank, targetdisp, targetcount, targettype, op, win);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ACCUMULATE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GET], [if MPI_Get is missing])
    AC_MSG_CHECKING([whether MPI_Get is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int origincount;
          MPI_Datatype origintype;
          int targetrank;
          MPI_Aint targetdisp;
          int targetcount;
          MPI_Datatype targettype;
          MPI_Win win;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Get(buf, origincount, origintype, targetrank, targetdisp, targetcount, targettype, win);
          PMPI_Get(buf, origincount, origintype, targetrank, targetdisp, targetcount, targettype, win);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GET, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_PUT], [if MPI_Put is missing])
    AC_MSG_CHECKING([whether MPI_Put is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int origincount;
          MPI_Datatype origintype;
          int targetrank;
          MPI_Aint targetdisp;
          int targetcount;
          MPI_Datatype targettype;
          MPI_Win win;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Put(buf, origincount, origintype, targetrank, targetdisp, targetcount, targettype, win);
          PMPI_Put(buf, origincount, origintype, targetrank, targetdisp, targetcount, targettype, win);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_PUT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_COMPLETE], [if MPI_Win_complete is missing])
    AC_MSG_CHECKING([whether MPI_Win_complete is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win win;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_complete(win);
          PMPI_Win_complete(win);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_COMPLETE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_CREATE], [if MPI_Win_create is missing])
    AC_MSG_CHECKING([whether MPI_Win_create is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *addr;
          MPI_Aint size;
          int dispunit;
          MPI_Info info;
          MPI_Comm comm;
          MPI_Win *win;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_create(addr, size, dispunit, info, comm, win);
          PMPI_Win_create(addr, size, dispunit, info, comm, win);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_CREATE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_FENCE], [if MPI_Win_fence is missing])
    AC_MSG_CHECKING([whether MPI_Win_fence is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int assertion;
          MPI_Win win;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_fence(assertion, win);
          PMPI_Win_fence(assertion, win);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_FENCE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_FREE], [if MPI_Win_free is missing])
    AC_MSG_CHECKING([whether MPI_Win_free is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win *win;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_free(win);
          PMPI_Win_free(win);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_FREE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_GET_GROUP], [if MPI_Win_get_group is missing])
    AC_MSG_CHECKING([whether MPI_Win_get_group is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win win;
          MPI_Group *group;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_get_group(win, group);
          PMPI_Win_get_group(win, group);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_GET_GROUP, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_LOCK], [if MPI_Win_lock is missing])
    AC_MSG_CHECKING([whether MPI_Win_lock is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int locktype;
          int winrank;
          int assertion;
          MPI_Win win;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_lock(locktype, winrank, assertion, win);
          PMPI_Win_lock(locktype, winrank, assertion, win);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_LOCK, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_POST], [if MPI_Win_post is missing])
    AC_MSG_CHECKING([whether MPI_Win_post is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Group group;
          int assertion;
          MPI_Win win;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_post(group, assertion, win);
          PMPI_Win_post(group, assertion, win);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_POST, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_START], [if MPI_Win_start is missing])
    AC_MSG_CHECKING([whether MPI_Win_start is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Group group;
          int assertion;
          MPI_Win win;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_start(group, assertion, win);
          PMPI_Win_start(group, assertion, win);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_START, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_TEST], [if MPI_Win_test is missing])
    AC_MSG_CHECKING([whether MPI_Win_test is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win win;
          int *flag;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_test(win, flag);
          PMPI_Win_test(win, flag);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_TEST, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_UNLOCK], [if MPI_Win_unlock is missing])
    AC_MSG_CHECKING([whether MPI_Win_unlock is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int winrank;
          MPI_Win win;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_unlock(winrank, win);
          PMPI_Win_unlock(winrank, win);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_UNLOCK, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_WAIT], [if MPI_Win_wait is missing])
    AC_MSG_CHECKING([whether MPI_Win_wait is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win win;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_wait(win);
          PMPI_Win_wait(win);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_WAIT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ALLTOALLW], [if MPI_Alltoallw is missing])
    AC_MSG_CHECKING([whether MPI_Alltoallw is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          int *sendcounts;
          int *senddispls;
          MPI_Datatype *sendtypes;
          void *recvbuf;
          int *recvcounts;
          int *recvdispls;
          MPI_Datatype *recvtypes;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Alltoallw(sendbuf, sendcounts, senddispls, sendtypes, recvbuf, recvcounts, recvdispls, recvtypes, comm);
          PMPI_Alltoallw(sendbuf, sendcounts, senddispls, sendtypes, recvbuf, recvcounts, recvdispls, recvtypes, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ALLTOALLW, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_EXSCAN], [if MPI_Exscan is missing])
    AC_MSG_CHECKING([whether MPI_Exscan is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *sendbuf;
          void *recvbuf;
          int count;
          MPI_Datatype datatype;
          MPI_Op op;
          MPI_Comm comm;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
          PMPI_Exscan(sendbuf, recvbuf, count, datatype, op, comm);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_EXSCAN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ADD_ERROR_CLASS], [if MPI_Add_error_class is missing])
    AC_MSG_CHECKING([whether MPI_Add_error_class is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *errorclass;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Add_error_class(errorclass);
          PMPI_Add_error_class(errorclass);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ADD_ERROR_CLASS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ADD_ERROR_CODE], [if MPI_Add_error_code is missing])
    AC_MSG_CHECKING([whether MPI_Add_error_code is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int errorclass;
          int *errorcode;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Add_error_code(errorclass, errorcode);
          PMPI_Add_error_code(errorclass, errorcode);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ADD_ERROR_CODE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ADD_ERROR_STRING], [if MPI_Add_error_string is missing])
    AC_MSG_CHECKING([whether MPI_Add_error_string is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int errorcode;
          char *errorstring;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Add_error_string(errorcode, errorstring);
          PMPI_Add_error_string(errorcode, errorstring);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ADD_ERROR_STRING, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_CALL_ERRHANDLER], [if MPI_Comm_call_errhandler is missing])
    AC_MSG_CHECKING([whether MPI_Comm_call_errhandler is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int errorcode;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_call_errhandler(comm, errorcode);
          PMPI_Comm_call_errhandler(comm, errorcode);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_CALL_ERRHANDLER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_CREATE_KEYVAL], [if MPI_Comm_create_keyval is missing])
    AC_MSG_CHECKING([whether MPI_Comm_create_keyval is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm_copy_attr_function *copyfunc;
          MPI_Comm_delete_attr_function *deletefunc;
          int *keyval;
          void *extrastate;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_create_keyval(copyfunc, deletefunc, keyval, extrastate);
          PMPI_Comm_create_keyval(copyfunc, deletefunc, keyval, extrastate);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_CREATE_KEYVAL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_DELETE_ATTR], [if MPI_Comm_delete_attr is missing])
    AC_MSG_CHECKING([whether MPI_Comm_delete_attr is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int keyval;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_delete_attr(comm, keyval);
          PMPI_Comm_delete_attr(comm, keyval);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_DELETE_ATTR, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_FREE_KEYVAL], [if MPI_Comm_free_keyval is missing])
    AC_MSG_CHECKING([whether MPI_Comm_free_keyval is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *keyval;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_free_keyval(keyval);
          PMPI_Comm_free_keyval(keyval);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_FREE_KEYVAL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_GET_ATTR], [if MPI_Comm_get_attr is missing])
    AC_MSG_CHECKING([whether MPI_Comm_get_attr is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int keyval;
          void *value;
          int *flag;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_get_attr(comm, keyval, value, flag);
          PMPI_Comm_get_attr(comm, keyval, value, flag);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_GET_ATTR, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_GET_NAME], [if MPI_Comm_get_name is missing])
    AC_MSG_CHECKING([whether MPI_Comm_get_name is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          char *name;
          int *resultlen;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_get_name(comm, name, resultlen);
          PMPI_Comm_get_name(comm, name, resultlen);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_GET_NAME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_SET_ATTR], [if MPI_Comm_set_attr is missing])
    AC_MSG_CHECKING([whether MPI_Comm_set_attr is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          int keyval;
          void *value;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_set_attr(comm, keyval, value);
          PMPI_Comm_set_attr(comm, keyval, value);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_SET_ATTR, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_SET_NAME], [if MPI_Comm_set_name is missing])
    AC_MSG_CHECKING([whether MPI_Comm_set_name is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          char *name;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_set_name(comm, name);
          PMPI_Comm_set_name(comm, name);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_SET_NAME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_CALL_ERRHANDLER], [if MPI_File_call_errhandler is missing])
    AC_MSG_CHECKING([whether MPI_File_call_errhandler is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          int errorcode;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_call_errhandler(file, errorcode);
          PMPI_File_call_errhandler(file, errorcode);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_CALL_ERRHANDLER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GREQUEST_COMPLETE], [if MPI_Grequest_complete is missing])
    AC_MSG_CHECKING([whether MPI_Grequest_complete is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Request request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Grequest_complete(request);
          PMPI_Grequest_complete(request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GREQUEST_COMPLETE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GREQUEST_START], [if MPI_Grequest_start is missing])
    AC_MSG_CHECKING([whether MPI_Grequest_start is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Grequest_query_function *queryfunc;
          MPI_Grequest_free_function *freefunc;
          MPI_Grequest_cancel_function *cancelfunc;
          void *state;
          MPI_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Grequest_start(queryfunc, freefunc, cancelfunc, state, request);
          PMPI_Grequest_start(queryfunc, freefunc, cancelfunc, state, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GREQUEST_START, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_IS_THREAD_MAIN], [if MPI_Is_thread_main is missing])
    AC_MSG_CHECKING([whether MPI_Is_thread_main is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *flag;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Is_thread_main(flag);
          PMPI_Is_thread_main(flag);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_IS_THREAD_MAIN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_QUERY_THREAD], [if MPI_Query_thread is missing])
    AC_MSG_CHECKING([whether MPI_Query_thread is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *supported;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Query_thread(supported);
          PMPI_Query_thread(supported);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_QUERY_THREAD, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_STATUS_SET_CANCELLED], [if MPI_Status_set_cancelled is missing])
    AC_MSG_CHECKING([whether MPI_Status_set_cancelled is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Status *status;
          int flag;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Status_set_cancelled(status, flag);
          PMPI_Status_set_cancelled(status, flag);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_STATUS_SET_CANCELLED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_STATUS_SET_ELEMENTS], [if MPI_Status_set_elements is missing])
    AC_MSG_CHECKING([whether MPI_Status_set_elements is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Status *status;
          MPI_Datatype datatype;
          int count;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Status_set_elements(status, datatype, count);
          PMPI_Status_set_elements(status, datatype, count);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_STATUS_SET_ELEMENTS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_CREATE_KEYVAL], [if MPI_Type_create_keyval is missing])
    AC_MSG_CHECKING([whether MPI_Type_create_keyval is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Type_copy_attr_function *copyfunc;
          MPI_Type_delete_attr_function *deletefunc;
          int *keyval;
          void *state;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_create_keyval(copyfunc, deletefunc, keyval, state);
          PMPI_Type_create_keyval(copyfunc, deletefunc, keyval, state);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_CREATE_KEYVAL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_DELETE_ATTR], [if MPI_Type_delete_attr is missing])
    AC_MSG_CHECKING([whether MPI_Type_delete_attr is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          int keyval;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_delete_attr(datatype, keyval);
          PMPI_Type_delete_attr(datatype, keyval);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_DELETE_ATTR, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_DUP], [if MPI_Type_dup is missing])
    AC_MSG_CHECKING([whether MPI_Type_dup is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype oldtype;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_dup(oldtype, newtype);
          PMPI_Type_dup(oldtype, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_DUP, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_FREE_KEYVAL], [if MPI_Type_free_keyval is missing])
    AC_MSG_CHECKING([whether MPI_Type_free_keyval is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *keyval;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_free_keyval(keyval);
          PMPI_Type_free_keyval(keyval);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_FREE_KEYVAL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_GET_ATTR], [if MPI_Type_get_attr is missing])
    AC_MSG_CHECKING([whether MPI_Type_get_attr is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          int keyval;
          void *value;
          int *flag;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_get_attr(datatype, keyval, value, flag);
          PMPI_Type_get_attr(datatype, keyval, value, flag);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_GET_ATTR, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_GET_CONTENTS], [if MPI_Type_get_contents is missing])
    AC_MSG_CHECKING([whether MPI_Type_get_contents is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          int maxintegers;
          int maxaddresses;
          int maxdatatypes;
          int *arrintegers;
          MPI_Aint *arraddresses;
          MPI_Datatype *arrdatatypes;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_get_contents(datatype, maxintegers, maxaddresses, maxdatatypes, arrintegers, arraddresses, arrdatatypes);
          PMPI_Type_get_contents(datatype, maxintegers, maxaddresses, maxdatatypes, arrintegers, arraddresses, arrdatatypes);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_GET_CONTENTS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_GET_ENVELOPE], [if MPI_Type_get_envelope is missing])
    AC_MSG_CHECKING([whether MPI_Type_get_envelope is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          int *numintegers;
          int *numaddresses;
          int *numdatatypes;
          int *combiner;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_get_envelope(datatype, numintegers, numaddresses, numdatatypes, combiner);
          PMPI_Type_get_envelope(datatype, numintegers, numaddresses, numdatatypes, combiner);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_GET_ENVELOPE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_GET_NAME], [if MPI_Type_get_name is missing])
    AC_MSG_CHECKING([whether MPI_Type_get_name is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          char *name;
          int *resultlen;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_get_name(datatype, name, resultlen);
          PMPI_Type_get_name(datatype, name, resultlen);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_GET_NAME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_SET_ATTR], [if MPI_Type_set_attr is missing])
    AC_MSG_CHECKING([whether MPI_Type_set_attr is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          int keyval;
          void *state;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_set_attr(datatype, keyval, state);
          PMPI_Type_set_attr(datatype, keyval, state);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_SET_ATTR, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_SET_NAME], [if MPI_Type_set_name is missing])
    AC_MSG_CHECKING([whether MPI_Type_set_name is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          char *name;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_set_name(datatype, name);
          PMPI_Type_set_name(datatype, name);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_SET_NAME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_MATCH_SIZE], [if MPI_Type_match_size is missing])
    AC_MSG_CHECKING([whether MPI_Type_match_size is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int typeclass;
          int size;
          MPI_Datatype *datatype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_match_size(typeclass, size, datatype);
          PMPI_Type_match_size(typeclass, size, datatype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_MATCH_SIZE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_CALL_ERRHANDLER], [if MPI_Win_call_errhandler is missing])
    AC_MSG_CHECKING([whether MPI_Win_call_errhandler is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win win;
          int errorcode;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_call_errhandler(win, errorcode);
          PMPI_Win_call_errhandler(win, errorcode);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_CALL_ERRHANDLER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_CREATE_KEYVAL], [if MPI_Win_create_keyval is missing])
    AC_MSG_CHECKING([whether MPI_Win_create_keyval is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win_copy_attr_function *copyfun;
          MPI_Win_delete_attr_function *deletefun;
          int *keyval;
          void *state;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_create_keyval(copyfun, deletefun, keyval, state);
          PMPI_Win_create_keyval(copyfun, deletefun, keyval, state);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_CREATE_KEYVAL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_DELETE_ATTR], [if MPI_Win_delete_attr is missing])
    AC_MSG_CHECKING([whether MPI_Win_delete_attr is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win win;
          int keyval;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_delete_attr(win, keyval);
          PMPI_Win_delete_attr(win, keyval);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_DELETE_ATTR, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_FREE_KEYVAL], [if MPI_Win_free_keyval is missing])
    AC_MSG_CHECKING([whether MPI_Win_free_keyval is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *keyval;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_free_keyval(keyval);
          PMPI_Win_free_keyval(keyval);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_FREE_KEYVAL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_GET_ATTR], [if MPI_Win_get_attr is missing])
    AC_MSG_CHECKING([whether MPI_Win_get_attr is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win win;
          int keyval;
          void *state;
          int *flag;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_get_attr(win, keyval, state, flag);
          PMPI_Win_get_attr(win, keyval, state, flag);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_GET_ATTR, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_GET_NAME], [if MPI_Win_get_name is missing])
    AC_MSG_CHECKING([whether MPI_Win_get_name is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win win;
          char *name;
          int *resultlen;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_get_name(win, name, resultlen);
          PMPI_Win_get_name(win, name, resultlen);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_GET_NAME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_SET_ATTR], [if MPI_Win_set_attr is missing])
    AC_MSG_CHECKING([whether MPI_Win_set_attr is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win win;
          int keyval;
          void *state;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_set_attr(win, keyval, state);
          PMPI_Win_set_attr(win, keyval, state);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_SET_ATTR, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_SET_NAME], [if MPI_Win_set_name is missing])
    AC_MSG_CHECKING([whether MPI_Win_set_name is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win win;
          char *name;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_set_name(win, name);
          PMPI_Win_set_name(win, name);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_SET_NAME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_ALLOC_MEM], [if MPI_Alloc_mem is missing])
    AC_MSG_CHECKING([whether MPI_Alloc_mem is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Aint size;
          MPI_Info info;
          void *baseptr;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Alloc_mem(size, info, baseptr);
          PMPI_Alloc_mem(size, info, baseptr);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_ALLOC_MEM, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_CREATE_ERRHANDLER], [if MPI_Comm_create_errhandler is missing])
    AC_MSG_CHECKING([whether MPI_Comm_create_errhandler is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm_errhandler_fn *errfun;
          MPI_Errhandler *errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_create_errhandler(errfun, errhandler);
          PMPI_Comm_create_errhandler(errfun, errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_CREATE_ERRHANDLER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_GET_ERRHANDLER], [if MPI_Comm_get_errhandler is missing])
    AC_MSG_CHECKING([whether MPI_Comm_get_errhandler is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          MPI_Errhandler *errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_get_errhandler(comm, errhandler);
          PMPI_Comm_get_errhandler(comm, errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_GET_ERRHANDLER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_COMM_SET_ERRHANDLER], [if MPI_Comm_set_errhandler is missing])
    AC_MSG_CHECKING([whether MPI_Comm_set_errhandler is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          MPI_Errhandler errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Comm_set_errhandler(comm, errhandler);
          PMPI_Comm_set_errhandler(comm, errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_COMM_SET_ERRHANDLER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_CREATE_ERRHANDLER], [if MPI_File_create_errhandler is missing])
    AC_MSG_CHECKING([whether MPI_File_create_errhandler is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File_errhandler_fn *errfun;
          MPI_Errhandler *errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_create_errhandler(errfun, errhandler);
          PMPI_File_create_errhandler(errfun, errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_CREATE_ERRHANDLER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_GET_ERRHANDLER], [if MPI_File_get_errhandler is missing])
    AC_MSG_CHECKING([whether MPI_File_get_errhandler is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Errhandler *errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_get_errhandler(file, errhandler);
          PMPI_File_get_errhandler(file, errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_GET_ERRHANDLER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_SET_ERRHANDLER], [if MPI_File_set_errhandler is missing])
    AC_MSG_CHECKING([whether MPI_File_set_errhandler is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Errhandler errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_set_errhandler(file, errhandler);
          PMPI_File_set_errhandler(file, errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_SET_ERRHANDLER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FINALIZED], [if MPI_Finalized is missing])
    AC_MSG_CHECKING([whether MPI_Finalized is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int *flag;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Finalized(flag);
          PMPI_Finalized(flag);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FINALIZED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FREE_MEM], [if MPI_Free_mem is missing])
    AC_MSG_CHECKING([whether MPI_Free_mem is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Free_mem(buf);
          PMPI_Free_mem(buf);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FREE_MEM, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_GET_ADDRESS], [if MPI_Get_address is missing])
    AC_MSG_CHECKING([whether MPI_Get_address is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          void *buf;
          MPI_Aint *address;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Get_address(buf, address);
          PMPI_Get_address(buf, address);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_GET_ADDRESS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INFO_CREATE], [if MPI_Info_create is missing])
    AC_MSG_CHECKING([whether MPI_Info_create is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Info *info;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Info_create(info);
          PMPI_Info_create(info);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INFO_CREATE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INFO_DELETE], [if MPI_Info_delete is missing])
    AC_MSG_CHECKING([whether MPI_Info_delete is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Info info;
          char *key;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Info_delete(info, key);
          PMPI_Info_delete(info, key);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INFO_DELETE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INFO_DUP], [if MPI_Info_dup is missing])
    AC_MSG_CHECKING([whether MPI_Info_dup is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Info oldinfo;
          MPI_Info *newinfo;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Info_dup(oldinfo, newinfo);
          PMPI_Info_dup(oldinfo, newinfo);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INFO_DUP, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INFO_FREE], [if MPI_Info_free is missing])
    AC_MSG_CHECKING([whether MPI_Info_free is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Info *info;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Info_free(info);
          PMPI_Info_free(info);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INFO_FREE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INFO_GET], [if MPI_Info_get is missing])
    AC_MSG_CHECKING([whether MPI_Info_get is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Info info;
          char *key;
          int valuelength;
          char *value;
          int *flag;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Info_get(info, key, valuelength, value, flag);
          PMPI_Info_get(info, key, valuelength, value, flag);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INFO_GET, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INFO_GET_NKEYS], [if MPI_Info_get_nkeys is missing])
    AC_MSG_CHECKING([whether MPI_Info_get_nkeys is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Info info;
          int *nkeys;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Info_get_nkeys(info, nkeys);
          PMPI_Info_get_nkeys(info, nkeys);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INFO_GET_NKEYS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INFO_GET_NTHKEY], [if MPI_Info_get_nthkey is missing])
    AC_MSG_CHECKING([whether MPI_Info_get_nthkey is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Info info;
          int n;
          char *key;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Info_get_nthkey(info, n, key);
          PMPI_Info_get_nthkey(info, n, key);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INFO_GET_NTHKEY, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INFO_GET_VALUELEN], [if MPI_Info_get_valuelen is missing])
    AC_MSG_CHECKING([whether MPI_Info_get_valuelen is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Info info;
          char *key;
          int *valuelen;
          int *flag;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Info_get_valuelen(info, key, valuelen, flag);
          PMPI_Info_get_valuelen(info, key, valuelen, flag);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INFO_GET_VALUELEN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_INFO_SET], [if MPI_Info_set is missing])
    AC_MSG_CHECKING([whether MPI_Info_set is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Info info;
          char *key;
          char *value;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Info_set(info, key, value);
          PMPI_Info_set(info, key, value);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_INFO_SET, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_PACK_EXTERNAL], [if MPI_Pack_external is missing])
    AC_MSG_CHECKING([whether MPI_Pack_external is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *datarep;
          void *inbuf;
          int incount;
          MPI_Datatype intype;
          void *outbuf;
          MPI_Aint outcount;
          MPI_Aint *position;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Pack_external(datarep, inbuf, incount, intype, outbuf, outcount, position);
          PMPI_Pack_external(datarep, inbuf, incount, intype, outbuf, outcount, position);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_PACK_EXTERNAL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_PACK_EXTERNAL_SIZE], [if MPI_Pack_external_size is missing])
    AC_MSG_CHECKING([whether MPI_Pack_external_size is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *datarep;
          int incount;
          MPI_Datatype datatype;
          MPI_Aint *size;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Pack_external_size(datarep, incount, datatype, size);
          PMPI_Pack_external_size(datarep, incount, datatype, size);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_PACK_EXTERNAL_SIZE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_REQUEST_GET_STATUS], [if MPI_Request_get_status is missing])
    AC_MSG_CHECKING([whether MPI_Request_get_status is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Request request;
          int *flag;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Request_get_status(request, flag, status);
          PMPI_Request_get_status(request, flag, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_REQUEST_GET_STATUS, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_CREATE_DARRAY], [if MPI_Type_create_darray is missing])
    AC_MSG_CHECKING([whether MPI_Type_create_darray is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int size;
          int rank;
          int ndims;
          int *gsizes;
          int *distribs;
          int *dargs;
          int *psizes;
          int order;
          MPI_Datatype oldtype;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_create_darray(size, rank, ndims, gsizes, distribs, dargs, psizes, order, oldtype, newtype);
          PMPI_Type_create_darray(size, rank, ndims, gsizes, distribs, dargs, psizes, order, oldtype, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_CREATE_DARRAY, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_CREATE_HINDEXED], [if MPI_Type_create_hindexed is missing])
    AC_MSG_CHECKING([whether MPI_Type_create_hindexed is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          int *blocklengths;
          MPI_Aint *displacements;
          MPI_Datatype oldtype;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_create_hindexed(count, blocklengths, displacements, oldtype, newtype);
          PMPI_Type_create_hindexed(count, blocklengths, displacements, oldtype, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_CREATE_HINDEXED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_CREATE_HVECTOR], [if MPI_Type_create_hvector is missing])
    AC_MSG_CHECKING([whether MPI_Type_create_hvector is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          int blocklength;
          MPI_Aint stride;
          MPI_Datatype oldtype;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_create_hvector(count, blocklength, stride, oldtype, newtype);
          PMPI_Type_create_hvector(count, blocklength, stride, oldtype, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_CREATE_HVECTOR, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_CREATE_INDEXED_BLOCK], [if MPI_Type_create_indexed_block is missing])
    AC_MSG_CHECKING([whether MPI_Type_create_indexed_block is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          int blocklength;
          int *displacments;
          MPI_Datatype oldtype;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_create_indexed_block(count, blocklength, displacments, oldtype, newtype);
          PMPI_Type_create_indexed_block(count, blocklength, displacments, oldtype, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_CREATE_INDEXED_BLOCK, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_CREATE_RESIZED], [if MPI_Type_create_resized is missing])
    AC_MSG_CHECKING([whether MPI_Type_create_resized is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype oldtype;
          MPI_Aint lb;
          MPI_Aint extent;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_create_resized(oldtype, lb, extent, newtype);
          PMPI_Type_create_resized(oldtype, lb, extent, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_CREATE_RESIZED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_CREATE_STRUCT], [if MPI_Type_create_struct is missing])
    AC_MSG_CHECKING([whether MPI_Type_create_struct is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          int *blocklengths;
          MPI_Aint *displacements;
          MPI_Datatype *oldtypes;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_create_struct(count, blocklengths, displacements, oldtypes, newtype);
          PMPI_Type_create_struct(count, blocklengths, displacements, oldtypes, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_CREATE_STRUCT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_CREATE_SUBARRAY], [if MPI_Type_create_subarray is missing])
    AC_MSG_CHECKING([whether MPI_Type_create_subarray is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int ndims;
          int *sizes;
          int *subsizes;
          int *starts;
          int order;
          MPI_Datatype oldtype;
          MPI_Datatype *newtype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_create_subarray(ndims, sizes, subsizes, starts, order, oldtype, newtype);
          PMPI_Type_create_subarray(ndims, sizes, subsizes, starts, order, oldtype, newtype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_CREATE_SUBARRAY, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_GET_EXTENT], [if MPI_Type_get_extent is missing])
    AC_MSG_CHECKING([whether MPI_Type_get_extent is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          MPI_Aint *lb;
          MPI_Aint *extent;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_get_extent(datatype, lb, extent);
          PMPI_Type_get_extent(datatype, lb, extent);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_GET_EXTENT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_TYPE_GET_TRUE_EXTENT], [if MPI_Type_get_true_extent is missing])
    AC_MSG_CHECKING([whether MPI_Type_get_true_extent is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Datatype datatype;
          MPI_Aint *lb;
          MPI_Aint *extent;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Type_get_true_extent(datatype, lb, extent);
          PMPI_Type_get_true_extent(datatype, lb, extent);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_TYPE_GET_TRUE_EXTENT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_UNPACK_EXTERNAL], [if MPI_Unpack_external is missing])
    AC_MSG_CHECKING([whether MPI_Unpack_external is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *datarep;
          void *inbuf;
          MPI_Aint insize;
          MPI_Aint *position;
          void *outbuf;
          int outcount;
          MPI_Datatype datatype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Unpack_external(datarep, inbuf, insize, position, outbuf, outcount, datatype);
          PMPI_Unpack_external(datarep, inbuf, insize, position, outbuf, outcount, datatype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_UNPACK_EXTERNAL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_CREATE_ERRHANDLER], [if MPI_Win_create_errhandler is missing])
    AC_MSG_CHECKING([whether MPI_Win_create_errhandler is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win_errhandler_fn *errfun;
          MPI_Errhandler *errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_create_errhandler(errfun, errhandler);
          PMPI_Win_create_errhandler(errfun, errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_CREATE_ERRHANDLER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_GET_ERRHANDLER], [if MPI_Win_get_errhandler is missing])
    AC_MSG_CHECKING([whether MPI_Win_get_errhandler is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win win;
          MPI_Errhandler *errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_get_errhandler(win, errhandler);
          PMPI_Win_get_errhandler(win, errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_GET_ERRHANDLER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_WIN_SET_ERRHANDLER], [if MPI_Win_set_errhandler is missing])
    AC_MSG_CHECKING([whether MPI_Win_set_errhandler is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Win win;
          MPI_Errhandler errhandler;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Win_set_errhandler(win, errhandler);
          PMPI_Win_set_errhandler(win, errhandler);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_WIN_SET_ERRHANDLER, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_OPEN], [if MPI_File_open is missing])
    AC_MSG_CHECKING([whether MPI_File_open is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_Comm comm;
          char *filename;
          int amode;
          MPI_Info info;
          MPI_File *file;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_open(comm, filename, amode, info, file);
          PMPI_File_open(comm, filename, amode, info, file);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_OPEN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_CLOSE], [if MPI_File_close is missing])
    AC_MSG_CHECKING([whether MPI_File_close is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File *file;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_close(file);
          PMPI_File_close(file);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_CLOSE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_DELETE], [if MPI_File_delete is missing])
    AC_MSG_CHECKING([whether MPI_File_delete is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *filename;
          MPI_Info info;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_delete(filename, info);
          PMPI_File_delete(filename, info);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_DELETE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_SET_SIZE], [if MPI_File_set_size is missing])
    AC_MSG_CHECKING([whether MPI_File_set_size is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset size;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_set_size(file, size);
          PMPI_File_set_size(file, size);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_SET_SIZE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_PREALLOCATE], [if MPI_File_preallocate is missing])
    AC_MSG_CHECKING([whether MPI_File_preallocate is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset size;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_preallocate(file, size);
          PMPI_File_preallocate(file, size);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_PREALLOCATE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_GET_SIZE], [if MPI_File_get_size is missing])
    AC_MSG_CHECKING([whether MPI_File_get_size is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset *size;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_get_size(file, size);
          PMPI_File_get_size(file, size);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_GET_SIZE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_GET_GROUP], [if MPI_File_get_group is missing])
    AC_MSG_CHECKING([whether MPI_File_get_group is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Group *group;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_get_group(file, group);
          PMPI_File_get_group(file, group);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_GET_GROUP, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_GET_AMODE], [if MPI_File_get_amode is missing])
    AC_MSG_CHECKING([whether MPI_File_get_amode is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          int *amode;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_get_amode(file, amode);
          PMPI_File_get_amode(file, amode);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_GET_AMODE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_SET_INFO], [if MPI_File_set_info is missing])
    AC_MSG_CHECKING([whether MPI_File_set_info is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Info info;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_set_info(file, info);
          PMPI_File_set_info(file, info);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_SET_INFO, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_GET_INFO], [if MPI_File_get_info is missing])
    AC_MSG_CHECKING([whether MPI_File_get_info is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Info *info;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_get_info(file, info);
          PMPI_File_get_info(file, info);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_GET_INFO, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_SET_VIEW], [if MPI_File_set_view is missing])
    AC_MSG_CHECKING([whether MPI_File_set_view is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset offset;
          MPI_Datatype hosttype;
          MPI_Datatype filetype;
          char *datarep;
          MPI_Info info;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_set_view(file, offset, hosttype, filetype, datarep, info);
          PMPI_File_set_view(file, offset, hosttype, filetype, datarep, info);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_SET_VIEW, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_GET_VIEW], [if MPI_File_get_view is missing])
    AC_MSG_CHECKING([whether MPI_File_get_view is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset *offset;
          MPI_Datatype *hosttype;
          MPI_Datatype *filetype;
          char *datarep;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_get_view(file, offset, hosttype, filetype, datarep);
          PMPI_File_get_view(file, offset, hosttype, filetype, datarep);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_GET_VIEW, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_READ_AT], [if MPI_File_read_at is missing])
    AC_MSG_CHECKING([whether MPI_File_read_at is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset offset;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_read_at(file, offset, buf, count, datatype, status);
          PMPI_File_read_at(file, offset, buf, count, datatype, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_READ_AT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_READ_AT_ALL], [if MPI_File_read_at_all is missing])
    AC_MSG_CHECKING([whether MPI_File_read_at_all is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset offset;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_read_at_all(file, offset, buf, count, datatype, status);
          PMPI_File_read_at_all(file, offset, buf, count, datatype, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_READ_AT_ALL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_WRITE_AT], [if MPI_File_write_at is missing])
    AC_MSG_CHECKING([whether MPI_File_write_at is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset offset;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_write_at(file, offset, buf, count, datatype, status);
          PMPI_File_write_at(file, offset, buf, count, datatype, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_WRITE_AT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_WRITE_AT_ALL], [if MPI_File_write_at_all is missing])
    AC_MSG_CHECKING([whether MPI_File_write_at_all is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset offset;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_write_at_all(file, offset, buf, count, datatype, status);
          PMPI_File_write_at_all(file, offset, buf, count, datatype, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_WRITE_AT_ALL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_IREAD_AT], [if MPI_File_iread_at is missing])
    AC_MSG_CHECKING([whether MPI_File_iread_at is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset offset;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPIO_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_iread_at(file, offset, buf, count, datatype, request);
          PMPI_File_iread_at(file, offset, buf, count, datatype, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_IREAD_AT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_IWRITE_AT], [if MPI_File_iwrite_at is missing])
    AC_MSG_CHECKING([whether MPI_File_iwrite_at is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset offset;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPIO_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_iwrite_at(file, offset, buf, count, datatype, request);
          PMPI_File_iwrite_at(file, offset, buf, count, datatype, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_IWRITE_AT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_READ], [if MPI_File_read is missing])
    AC_MSG_CHECKING([whether MPI_File_read is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_read(file, buf, count, datatype, status);
          PMPI_File_read(file, buf, count, datatype, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_READ, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_READ_ALL], [if MPI_File_read_all is missing])
    AC_MSG_CHECKING([whether MPI_File_read_all is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_read_all(file, buf, count, datatype, status);
          PMPI_File_read_all(file, buf, count, datatype, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_READ_ALL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_WRITE], [if MPI_File_write is missing])
    AC_MSG_CHECKING([whether MPI_File_write is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_write(file, buf, count, datatype, status);
          PMPI_File_write(file, buf, count, datatype, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_WRITE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_WRITE_ALL], [if MPI_File_write_all is missing])
    AC_MSG_CHECKING([whether MPI_File_write_all is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_write_all(file, buf, count, datatype, status);
          PMPI_File_write_all(file, buf, count, datatype, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_WRITE_ALL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_IREAD], [if MPI_File_iread is missing])
    AC_MSG_CHECKING([whether MPI_File_iread is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPIO_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_iread(file, buf, count, datatype, request);
          PMPI_File_iread(file, buf, count, datatype, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_IREAD, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_IWRITE], [if MPI_File_iwrite is missing])
    AC_MSG_CHECKING([whether MPI_File_iwrite is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPIO_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_iwrite(file, buf, count, datatype, request);
          PMPI_File_iwrite(file, buf, count, datatype, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_IWRITE, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_SEEK], [if MPI_File_seek is missing])
    AC_MSG_CHECKING([whether MPI_File_seek is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset offset;
          int whence;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_seek(file, offset, whence);
          PMPI_File_seek(file, offset, whence);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_SEEK, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_GET_POSITION], [if MPI_File_get_position is missing])
    AC_MSG_CHECKING([whether MPI_File_get_position is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset *offset;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_get_position(file, offset);
          PMPI_File_get_position(file, offset);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_GET_POSITION, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_GET_BYTE_OFFSET], [if MPI_File_get_byte_offset is missing])
    AC_MSG_CHECKING([whether MPI_File_get_byte_offset is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset offset;
          MPI_Offset *bytes;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_get_byte_offset(file, offset, bytes);
          PMPI_File_get_byte_offset(file, offset, bytes);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_GET_BYTE_OFFSET, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_READ_SHARED], [if MPI_File_read_shared is missing])
    AC_MSG_CHECKING([whether MPI_File_read_shared is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_read_shared(file, buf, count, datatype, status);
          PMPI_File_read_shared(file, buf, count, datatype, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_READ_SHARED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_WRITE_SHARED], [if MPI_File_write_shared is missing])
    AC_MSG_CHECKING([whether MPI_File_write_shared is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_write_shared(file, buf, count, datatype, status);
          PMPI_File_write_shared(file, buf, count, datatype, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_WRITE_SHARED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_IREAD_SHARED], [if MPI_File_iread_shared is missing])
    AC_MSG_CHECKING([whether MPI_File_iread_shared is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPIO_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_iread_shared(file, buf, count, datatype, request);
          PMPI_File_iread_shared(file, buf, count, datatype, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_IREAD_SHARED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_IWRITE_SHARED], [if MPI_File_iwrite_shared is missing])
    AC_MSG_CHECKING([whether MPI_File_iwrite_shared is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPIO_Request *request;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_iwrite_shared(file, buf, count, datatype, request);
          PMPI_File_iwrite_shared(file, buf, count, datatype, request);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_IWRITE_SHARED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_READ_ORDERED], [if MPI_File_read_ordered is missing])
    AC_MSG_CHECKING([whether MPI_File_read_ordered is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_read_ordered(file, buf, count, datatype, status);
          PMPI_File_read_ordered(file, buf, count, datatype, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_READ_ORDERED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_WRITE_ORDERED], [if MPI_File_write_ordered is missing])
    AC_MSG_CHECKING([whether MPI_File_write_ordered is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_write_ordered(file, buf, count, datatype, status);
          PMPI_File_write_ordered(file, buf, count, datatype, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_WRITE_ORDERED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_SEEK_SHARED], [if MPI_File_seek_shared is missing])
    AC_MSG_CHECKING([whether MPI_File_seek_shared is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset offset;
          int whence;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_seek_shared(file, offset, whence);
          PMPI_File_seek_shared(file, offset, whence);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_SEEK_SHARED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_GET_POSITION_SHARED], [if MPI_File_get_position_shared is missing])
    AC_MSG_CHECKING([whether MPI_File_get_position_shared is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset *offset;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_get_position_shared(file, offset);
          PMPI_File_get_position_shared(file, offset);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_GET_POSITION_SHARED, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_READ_AT_ALL_BEGIN], [if MPI_File_read_at_all_begin is missing])
    AC_MSG_CHECKING([whether MPI_File_read_at_all_begin is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset offset;
          void *buf;
          int count;
          MPI_Datatype datatype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_read_at_all_begin(file, offset, buf, count, datatype);
          PMPI_File_read_at_all_begin(file, offset, buf, count, datatype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_READ_AT_ALL_BEGIN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_READ_AT_ALL_END], [if MPI_File_read_at_all_end is missing])
    AC_MSG_CHECKING([whether MPI_File_read_at_all_end is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_read_at_all_end(file, buf, status);
          PMPI_File_read_at_all_end(file, buf, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_READ_AT_ALL_END, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_WRITE_AT_ALL_BEGIN], [if MPI_File_write_at_all_begin is missing])
    AC_MSG_CHECKING([whether MPI_File_write_at_all_begin is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Offset offset;
          void *buf;
          int count;
          MPI_Datatype datatype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_write_at_all_begin(file, offset, buf, count, datatype);
          PMPI_File_write_at_all_begin(file, offset, buf, count, datatype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_WRITE_AT_ALL_BEGIN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_WRITE_AT_ALL_END], [if MPI_File_write_at_all_end is missing])
    AC_MSG_CHECKING([whether MPI_File_write_at_all_end is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_write_at_all_end(file, buf, status);
          PMPI_File_write_at_all_end(file, buf, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_WRITE_AT_ALL_END, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_READ_ALL_BEGIN], [if MPI_File_read_all_begin is missing])
    AC_MSG_CHECKING([whether MPI_File_read_all_begin is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_read_all_begin(file, buf, count, datatype);
          PMPI_File_read_all_begin(file, buf, count, datatype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_READ_ALL_BEGIN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_READ_ALL_END], [if MPI_File_read_all_end is missing])
    AC_MSG_CHECKING([whether MPI_File_read_all_end is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_read_all_end(file, buf, status);
          PMPI_File_read_all_end(file, buf, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_READ_ALL_END, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_WRITE_ALL_BEGIN], [if MPI_File_write_all_begin is missing])
    AC_MSG_CHECKING([whether MPI_File_write_all_begin is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_write_all_begin(file, buf, count, datatype);
          PMPI_File_write_all_begin(file, buf, count, datatype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_WRITE_ALL_BEGIN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_WRITE_ALL_END], [if MPI_File_write_all_end is missing])
    AC_MSG_CHECKING([whether MPI_File_write_all_end is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_write_all_end(file, buf, status);
          PMPI_File_write_all_end(file, buf, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_WRITE_ALL_END, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_READ_ORDERED_BEGIN], [if MPI_File_read_ordered_begin is missing])
    AC_MSG_CHECKING([whether MPI_File_read_ordered_begin is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_read_ordered_begin(file, buf, count, datatype);
          PMPI_File_read_ordered_begin(file, buf, count, datatype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_READ_ORDERED_BEGIN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_READ_ORDERED_END], [if MPI_File_read_ordered_end is missing])
    AC_MSG_CHECKING([whether MPI_File_read_ordered_end is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_read_ordered_end(file, buf, status);
          PMPI_File_read_ordered_end(file, buf, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_READ_ORDERED_END, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_WRITE_ORDERED_BEGIN], [if MPI_File_write_ordered_begin is missing])
    AC_MSG_CHECKING([whether MPI_File_write_ordered_begin is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          int count;
          MPI_Datatype datatype;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_write_ordered_begin(file, buf, count, datatype);
          PMPI_File_write_ordered_begin(file, buf, count, datatype);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_WRITE_ORDERED_BEGIN, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_WRITE_ORDERED_END], [if MPI_File_write_ordered_end is missing])
    AC_MSG_CHECKING([whether MPI_File_write_ordered_end is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          void *buf;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_write_ordered_end(file, buf, status);
          PMPI_File_write_ordered_end(file, buf, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_WRITE_ORDERED_END, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_GET_TYPE_EXTENT], [if MPI_File_get_type_extent is missing])
    AC_MSG_CHECKING([whether MPI_File_get_type_extent is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          MPI_Datatype datatype;
          MPI_Aint *extent;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_get_type_extent(file, datatype, extent);
          PMPI_File_get_type_extent(file, datatype, extent);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_GET_TYPE_EXTENT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_REGISTER_DATAREP], [if MPI_Register_datarep is missing])
    AC_MSG_CHECKING([whether MPI_Register_datarep is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          char *name;
          MPI_Datarep_conversion_function *readfun;
          MPI_Datarep_conversion_function *writefun;
          MPI_Datarep_extent_function *extentfun;
          void *state;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_Register_datarep(name, readfun, writefun, extentfun, state);
          PMPI_Register_datarep(name, readfun, writefun, extentfun, state);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_REGISTER_DATAREP, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_SET_ATOMICITY], [if MPI_File_set_atomicity is missing])
    AC_MSG_CHECKING([whether MPI_File_set_atomicity is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          int flag;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_set_atomicity(file, flag);
          PMPI_File_set_atomicity(file, flag);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_SET_ATOMICITY, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_GET_ATOMICITY], [if MPI_File_get_atomicity is missing])
    AC_MSG_CHECKING([whether MPI_File_get_atomicity is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          int *flag;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_get_atomicity(file, flag);
          PMPI_File_get_atomicity(file, flag);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_GET_ATOMICITY, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPI_FILE_SYNC], [if MPI_File_sync is missing])
    AC_MSG_CHECKING([whether MPI_File_sync is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPI_File file;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPI_File_sync(file);
          PMPI_File_sync(file);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPI_FILE_SYNC, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPIO_TEST], [if MPIO_Test is missing])
    AC_MSG_CHECKING([whether MPIO_Test is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPIO_Request *request;
          int *flag;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPIO_Test(request, flag, status);
          PMPIO_Test(request, flag, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPIO_TEST, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPIO_WAIT], [if MPIO_Wait is missing])
    AC_MSG_CHECKING([whether MPIO_Wait is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          MPIO_Request *request;
          MPI_Status *status;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPIO_Wait(request, status);
          PMPIO_Wait(request, status);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPIO_WAIT, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPIO_TESTALL], [if MPIO_Testall is missing])
    AC_MSG_CHECKING([whether MPIO_Testall is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPIO_Request *requests;
          int *flag;
          MPI_Status *statuses;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPIO_Testall(count, requests, flag, statuses);
          PMPIO_Testall(count, requests, flag, statuses);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPIO_TESTALL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPIO_WAITALL], [if MPIO_Waitall is missing])
    AC_MSG_CHECKING([whether MPIO_Waitall is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPIO_Request *requests;
          MPI_Status *statuses;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPIO_Waitall(count, requests, statuses);
          PMPIO_Waitall(count, requests, statuses);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPIO_WAITALL, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPIO_TESTANY], [if MPIO_Testany is missing])
    AC_MSG_CHECKING([whether MPIO_Testany is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPIO_Request *requests;
          int *index;
          int *flag;
          MPI_Status *statuses;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPIO_Testany(count, requests, index, flag, statuses);
          PMPIO_Testany(count, requests, index, flag, statuses);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPIO_TESTANY, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPIO_WAITANY], [if MPIO_Waitany is missing])
    AC_MSG_CHECKING([whether MPIO_Waitany is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPIO_Request *requests;
          int *index;
          MPI_Status *statuses;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPIO_Waitany(count, requests, index, statuses);
          PMPIO_Waitany(count, requests, index, statuses);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPIO_WAITANY, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPIO_WAITSOME], [if MPIO_Waitsome is missing])
    AC_MSG_CHECKING([whether MPIO_Waitsome is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPIO_Request *requests;
          int *outcount;
          int *indices;
          MPI_Status *statuses;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPIO_Waitsome(count, requests, outcount, indices, statuses);
          PMPIO_Waitsome(count, requests, outcount, indices, statuses);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPIO_WAITSOME, [1])
      ]
    )

    AH_TEMPLATE([DUMPI_SKIP_MPIO_TESTSOME], [if MPIO_Testsome is missing])
    AC_MSG_CHECKING([whether MPIO_Testsome is present])
    AC_LINK_IFELSE(
      [AC_LANG_PROGRAM(
        [
          #include <mpi.h>
        ], [
          int count;
          MPIO_Request *requests;
          int *outcount;
          int *indices;
          MPI_Status *statuses;
          int argc;
          char **argv;
          MPI_Init(&argc, &argv);
          MPIO_Testsome(count, requests, outcount, indices, statuses);
          PMPIO_Testsome(count, requests, outcount, indices, statuses);
          MPI_Finalize();
        ])],
      [
        AC_MSG_RESULT([yes])
      ], [
        AC_MSG_RESULT([no])
        AC_DEFINE(DUMPI_SKIP_MPIO_TESTSOME, [1])
      ]
    )

    AC_LANG_POP
  ]
)
